!  Program by Norman Brenner from the basic algorithm by Charles Rader, May 1967.
!  The idea for the bit reversal was suggested by Ralph Alter. [Brenner, Rader, Alter of MIT Lincoln Labs]
!  Adapted from the work of James W. Cooley & John W. Tukey.
!  References:
!     -J.W. Cooley and J.W. Tukey: "An Algorithm for the Machine Calculation
!     of Complex Fourier Series", Math. Comput. 19, 90 (April 1965), pp.297-301.
!     -C. M. Rader and N. M. Brenner, "A new principle for fast Fourier transformation,"
!     IEEE Transactions on Acoustics, Speech, and Signal Processing, vol. ASSP-24, 
!     pp. 264-266, June 1976.
!     -IEEE Audio Transactions (June 1967), Special Issue on the FFT  

! ***********************************************************************************


!   REVISIONS by Norman B. Nill (MITRE Corp.), circa 2002,
!   for specific application of computing FFT of 2-D input image:
!   	Code converted from F66/F77 to Fortran-90 (initially using Alan Miller program);
!       then Fortran-90 code was cleaned-up & brought into basic conformance with Fortran-95.
!       Revised code to get rid of many of the "GO TOs", but over 40 remain (more difficult to remove). 
!	Subroutine* name changed from FOURT() to FFT() 
!	Changed IF constant to IFQ
!	Set dimensionality to 2-D (NDIM=2)
!       Set Fourier direction to Forward FFT (IFRWD=1) 
!       With dimensionality and FFT direction as constants, several code segments
!       could be shortened/changed/deleted, and this was done.
!	There are now 4 input arrays: datar, datai, workr, worki, and 2 input parameters:
!       	xlength, ylength, are defined in an external Module 
!		the 4 arrays' memorys are allocated in the subroutine that calls FFT().
!		Added double precision for sine, cosine & associated parameters.
!       THE ABOVE REVISIONS HAVE THE EFFECT OF CONSTRICTING FFT() SUCH THAT IT
!       WILL NOW ONLY COMPUTE FFT OF A REAL, 2-DIMENSIONAL INPUT DATA SET

!     *In January 1988 C.Potter (NCSA) modified Brenner's FOURT subroutine to 
!       a FUNCTON and, apparently, also changed the DATAR, DATAI multi-dimensional
!       arrays to one-dimensional arrays (regardless of input data dimensionality).
!       I changed Potter's Function back to a subroutine, but left DATAR, DATAI
!       as one-dimensional arrays. e.g., for 2-D image input, the DATAR input array contains
!       the pixel gray level values: top row left to right, next row left to right,...
!	The DATAR, DATAI output arrays are the real & imaginary parts of the FFT, respectively.

!   This FFT routine is supposed to work on square or rectangle image, but its use in this program  has been 
!   constrained to any size Square image only (rectangle FFT has some further difficulties that would need to be worked out)
!   So, only image width is used, but height parameter is included in case future version is extended
!   to compute FFT of rectangle image (currently height is set equal to width).



! ***********************************************************************************
!     Original June 1967 COMMENTS by N.Brenner:
!     (but note that not all of these original code comments apply to this revised code)

!     THE COOLEY-TUKEY FAST FOURIER TRANSFORM IN USASI BASIC FORTRAN
!     EVALUATES COMPLEX FOURIER SERIES FOR COMPLEX OR REAL FUNCTIONS.
!     THAT IS, IT COMPUTES
!     FTRAN(J1,J2,...) = SUM(DATA(I1,I2,...)*W1**(I1-1)*(J1-1)
!                                         *W2**(I2-1)*(J2-1)*...),
!     WHERE W1 = EXP(-2*PI*SQRT(-1)/NN(1)), W2 = EXP(-2*PI*SQRT(-1)/NN(2)),
!     ETC. AND I1 AND J1 RUN FROM 1 TO NN(1), I2 AND J2 RUN FROM 1 TO
!     NN(2), ETC.  THERE IS NO LIMIT ON THE DIMENSIONALITY (NUMBER OF
!     SUBSCRIPTS) OF THE ARRAY OF DATA.  THE PROGRAM WILL PERFORM
!     A THREE-DIMENSIONAL FOURIER TRANSFORM AS EASILY AS A ONE-DIMEN-
!     SIONAL ONE, THO IN A PROPORTIONATELY GREATER TIME.  AN INVERSE
!     TRANSFORM CAN BE PERFORMED, IN WHICH THE SIGN IN THE EXPONENTIALS
!     IS +, INSTEAD OF -.  IF AN INVERSE TRANSFORM IS PERFORMED UPON
!     AN ARRAY OF TRANSFORMED DATA, THE ORIGINAL DATA WILL REAPPEAR,
!     MULTIPLIED BY NN(1)*NN(2)*...  THE ARRAY OF INPUT DATA MAY BE
!     REAL OR COMPLEX, AT THE PROGRAMMERS OPTION, WITH A SAVING OF
!     ABOUT THIRTY PER CENT IN RUNNING TIME FOR REAL OVER COMPLEX.
!     (FOR FASTEST TRANSFORM OF REAL DATA, NN(1) SHOULD BE EVEN.)
!     THE TRANSFORM VALUES ARE ALWAYS COMPLEX, AND ARE RETURNED IN THE
!     ORIGINAL ARRAY OF DATA, REPLACING THE INPUT DATA.  

!     ***THE LENGTH OF EACH DIMENSION OF THE DATA ARRAY MAY BE ANY INTEGER. 

!     THE PROGRAM RUNS FASTER ON COMPOSITE INTEGERS THAN ON PRIMES, AND IS
!     PARTICULARLY FAST ON NUMBERS RICH IN FACTORS OF TWO.
!     TIMING IS IN FACT GIVEN BY THE FOLLOWING FORMULA.  LET NTOT BE THE
!     TOTAL NUMBER OF POINTS (REAL OR COMPLEX) IN THE DATA ARRAY, THAT
!     IS, NTOT = NN(1)*NN(2)*...  DECOMPOSE NTOT INTO ITS PRIME FACTORS,
!     SUCH AS 2**K2 * 3**K3 * 5**K5 * ...  LET SUM2 BE THE SUM OF ALL
!     THE FACTORS OF TWO IN NTOT, THAT IS, SUM2  =  2*K2.  LET SUMF BE
!     THE SUM OF ALL OTHER FACTORS OF NTOT, THAT IS, SUMF  =  3*K3+5*K5+..
!     THE TIME TAKEN BY A MULTIDIMENSIONAL TRANSFORM ON THESE NTOT DATA
!     IS T  =  T0 + T1*NTOT + T2*NTOT*SUM2 + T3*NTOT*SUMF.  FOR THE PAR-
!     TICULAR IMPLEMENTATION FORTRAN 32 ON THE CDC 3300 (FLOATING POINT
!     ADD TIME  =  SIX MICROSECONDS),
!     T  =  3000 + 600*NTOT + 50*NTOT*SUM2 + 175*NTOT*SUMF MICROSECONDS
!     ON COMPLEX DATA.

!     IMPLEMENTATION OF THE DEFINITION BY SUMMATION WILL RUN IN A TIME
!     PROPORTIONAL TO NTOT**2.  FOR HIGHLY COMPOSITE NTOT, THE SAVINGS
!     OFFERED BY COOLEY-TUKEY CAN BE DRAMATIC.  A MATRIX 100 BY 100 WILL
!     BE TRANSFORMED IN TIME PROPORTIONAL TO 10000*(2+2+2+2+5+5+5+5)  = 
!     280,000 (ASSUMING T2 AND T3 TO BE ROUGHLY COMPARABLE) VERSUS
!     10000**2  =  100,000,000 FOR THE STRAIGHTFORWARD TECHNIQUE.

!     THE COOLEY-TUKEY ALGORITHM PLACES TWO RESTRICTIONS UPON THE
!     NATURE OF THE DATA BEYOND THE USUAL RESTRICTION THAT
!     THE DATA FROM ONE CYCLE OF A PERIODIC FUNCTION.  THEY ARE--
!     1.  THE NUMBER OF INPUT DATA AND THE NUMBER OF TRANSFORM VALUES
!     MUST BE THE SAME.
!     2. CONSIDERING THE DATA TO BE IN THE TIME DOMAIN,
!     THEY MUST BE EQUI-SPACED AT INTERVALS OF DT.  FURTHER, THE TRANS-
!     FORM VALUES, CONSIDERED TO BE IN FREQUENCY SPACE, WILL BE EQUI-
!     SPACED FROM 0 TO 2*PI*(NN(I)-1)/(NN(I)*DT) AT INTERVALS OF
!     2*PI/(NN(I)*DT) FOR EACH DIMENSION OF LENGTH NN(I).  OF COURSE,
!     DT NEED NOT BE THE SAME FOR EVERY DIMENSION.

!     THE CALLING SEQUENCE IS--
!     CALL FOURT(DATAR,DATAI,NN,NDIM,IFRWD,ICPLX,WORKR,WORKI)

!     DATAR AND DATAI ARE THE ARRAYS USED TO HOLD THE REAL AND IMAGINARY
!     PARTS OF THE INPUT DATA ON INPUT AND THE TRANSFORM VALUES ON
!     OUTPUT.  THEY ARE FLOATING POINT ARRAYS, MULTIDIMENSIONAL WITH
!     IDENTICAL DIMENSIONALITY AND EXTENT.  THE EXTENT OF EACH DIMENSION
!     IS GIVEN IN THE INTEGER ARRAY NN, OF LENGTH NDIM.  THAT IS,
!     NDIM IS THE DIMENSIONALITY OF THE ARRAYS DATAR AND DATAI.
!     IFRWD IS AN INTEGER USED TO INDICATE THE DIRECTION OF THE FOURIER
!     TRANSFORM.  IT IS NON-ZERO TO INDICATE A FORWARD TRANSFORM
!     (EXPONENTIAL SIGN IS -) AND ZERO TO INDICATE AN INVERSE TRANSFORM
!     (SIGN IS +).  ICPLX IS AN INTEGER TO INDICATE WHETHER THE DATA
!     ARE REAL OR COMPLEX.  IT IS NON-ZERO FOR COMPLEX, ZERO FOR REAL.
!     IF IT IS ZERO (REAL) THE CONTENTS OF ARRAY DATAI WILL BE ASSUMED
!     TO BE ZERO, AND NEED NOT BE EXPLICITLY SET TO ZERO.  AS EXPLAINED
!     ABOVE, THE TRANSFORM RESULTS ARE ALWAYS COMPLEX AND ARE STORED
!     IN DATAR AND DATAI ON RETURN.  WORKR AND WORKI ARE ARRAYS USED
!     FOR WORKING STORAGE.  THEY ARE NOT NECESSARY IF ALL THE DIMENSIONS
!     OF THE DATA ARE POWERS OF TWO.  IN THIS CASE, THE ARRAYS MAY BE
!     REPLACED BY THE NUMBER 0 IN THE CALLING SEQUENCE.  THUS, USE OF
!     POWERS OF TWO CAN FREE A GOOD DEAL OF STORAGE.  IF ANY DIMENSION
!     IS NOT A POWER OF TWO, THESE ARRAYS MUST BE SUPPLIED.  THEY ARE
!     FLOATING POINT, ONE DIMENSIONAL OF LENGTH EQUAL TO THE LARGEST
!     ARRAY DIMENSION, THAT IS, TO THE LARGEST VALUE OF NN(I).
!     WORKR AND WORKI, IF SUPPLIED, MUST NOT BE THE SAME ARRAYS AS DATAR
!     OR DATAI.  ALL SUBSCRIPTS OF ALL ARRAYS BEGIN AT 1.

!     EXAMPLE 1.  THREE-DIMENSIONAL FORWARD FOURIER TRANSFORM OF A
!     COMPLEX ARRAY DIMENSIONED 100 BY 16 BY 13.
!     DIMENSION DATAR(100,16,13),DATAI(100,16,13),WORKR(100),WORKI(100)
!     DIMENSION NN(3)
!     NN(1) = 100
!     NN(2) = 16
!     NN(3) = 13
!     CALL FOURT(DATAR,DATAI,NN,3,1,1,WORKR,WORKI)

!     EXAMPLE 2.  ONE-DIMENSIONAL FORWARD TRANSFORM OF A REAL ARRAY OF
!     LENGTH 64.
!     DIMENSION DATAR(64),DATAI(64)
!     CALL FOURT(DATAR,DATAI,64,1,1,0,0,0)


!     THERE ARE NO ERROR MESSAGES OR ERROR HALTS IN THIS PROGRAM.  THE
!     PROGRAM RETURNS IMMEDIATELY IF NDIM OR ANY NN(I) IS LESS THAN ONE.

!     THE SINE AND COSINE VALUES REQUIRED FOR THE TRANSFORM ARE
!     GENERATED RECURSIVELY.  IF DOUBLE PRECISION IS AVAILABLE, IT IS
!     STRONGLY URGED THAT THE FOLLOWING VARIABLES BE SO DECLARED TO
!     REDUCE ACCUMULATION OF ROUNDOFF ERROR--

!     IN ADDITION, TWOPI SHOULD BE ASSIGNED A SUFFICIENTLY PRECISE
!     VALUE AND THE VARIOUS CALLS TO THE FUNCTIONS COS AND SIN
!     SHOULD BE CHANGED TO DCOS AND DSIN.

!     PROGRAM BY NORMAN BRENNER FROM THE BASIC ALGORITHM BY CHARLES
!     RADER (BOTH OF MIT LINCOLN LABORATORY).  MAY 1967.  THE IDEA
!     FOR THE BIT REVERSAL WAS SUGGESTED BY RALPH ALTER (ALSO MIT LL).
!     ADAPTED FROM THE WORK OF JAMES W. COOLEY AND JOHN W. TUKEY,
!     AN ALGORITHM FOR THE MACHINE CALCULATION OF COMPLEX FOURIER
!     SERIES, MATH. COMPUT. 19, 90 (APRIL 1965), 297-301.
! ***********************************************************************************

SUBROUTINE FFT()

USE GLOBALDATA

! Before entry into FFT(), Memory must be allocated for DATAR(:), DATAI(:), WORKR(:), WORKI(:) 
! and values for xlength, ylength must be defined.

DIMENSION  ifact(20), NN(2)

DOUBLE PRECISION :: twopi,theta,wstpr,wstpi,wminr,wmini,wr,wi,wtemp,  &
    		    thetm,wmstr,wmsti,twowr,sr,si,oldsr,oldsi,stmpr,stmpi


 NDIM = 2		! 2-D FFT of 2-D image array
 IFRWD = 1		! perform forward FFT
 ICPLX = 0		! imaginary part of input image data is all zero
 NN(1) = xlength	! image width in pixels
 NN(2) = ylength	! image height in pixels
 DATAI = 0.0
 WORKI = 0.0
 WORKR = 0.0
 ifact = 0		! dimension n of ifact(n) gives largest FFT of square image 
 			! that can be handled, i.e., largest image width for FFT is:
			! width = 2**(n+1) 
 twopi = 6.28318530717958647D0
 
 ntot = xlength * ylength


!     MAIN LOOP FOR EACH DIMENSION
np1 = 1
DO  idim = 1,  ndim
	n = NN(idim) 
  	np2 = np1 * n
        IF(n < 2) RETURN	! don't compute if image width or height < 2 pixels
			
!     IS N A POWER OF TWO AND IF NOT, WHAT ARE ITS FACTORS
 	m = n
  	ntwo = np1
  	IFQ = 1
  	idiv = 2
10      iquot = m/idiv
  	irem = m-idiv * iquot
	IF(iquot-idiv < 0) GOTO 50
        IF(irem  /=  0)  GOTO 20
        ntwo = ntwo +  ntwo
  	ifact(IFQ)  = idiv
  	IFQ = IFQ +  1
  	m = iquot
  	GOTO 10
20      idiv = 3
  	inon2 = IFQ
30      iquot = m/idiv
  	irem = m-idiv * iquot
	IF(iquot-idiv < 0) GOTO 60
        IF(irem  /=  0)  GOTO 40
        ifact(IFQ)  = idiv
  	IFQ = IFQ +  1
  	m = iquot
  	GOTO 30
40      idiv = idiv +  2
  	GOTO 30
50      inon2 = IFQ
  	IF(irem /= 0) GOTO 60
        ntwo = ntwo +  ntwo
  	GOTO 70
60      ifact(IFQ)  = m
70      non2p = np2/ntwo
  
  
!     SEPARATE FOUR CASES ("icase")
!        1. COMPLEX TRANSFORM
!        2. REAL TRANSFORM FOR THE 2ND,   3RD,   ETC. DIMENSION.  METHOD--
!           TRANSFORM HALF THE DATA,   SUPPLYING THE OTHER HALF BY CON-
!           JUGATE SYMMETRY.
!        3. REAL TRANSFORM FOR THE 1ST DIMENSION,   N ODD.  METHOD--
!           SET THE IMAGINARY PARTS TO ZERO.
!        4. REAL TRANSFORM FOR THE 1ST DIMENSION,   N EVEN.  METHOD--
!           TRANSFORM A COMPLEX ARRAY OF LENGTH N/2 WHOSE REAL PARTS
!           ARE THE EVEN NUMBERED REAL VALUES AND WHOSE IMAGINARY PARTS
!           ARE THE ODD-NUMBERED REAL VALUES.  UNSCRAMBLE AND SUPPLY
!           THE SECOND HALF BY CONJUGATE SYMMETRY.
  	icase = 1
  	ifmin = 1
    	icase = 2
	IF(idim-1 > 0) GOTO 100
    	icase = 3
	IF(ntwo-np1 <= 0) GOTO 100
    	icase = 4
  	ifmin = 2
  	ntwo = ntwo/2
  	n = n/2
  	np2 = np2/2
  	ntot = ntot/2
  	i = 1
  	DO  j = 1,  ntot
    		datar(j)  = datar(i) 
    		datai(j)  = datar(i +  1) 
    		i = i +  2
  	END DO
  
!     SHUFFLE DATA BY BIT REVERSAL,   SINCE N = 2 *  * K.  AS THE SHUFFLING
!     CAN BE DONE BY SIMPLE INTERCHANGE,   NO WORKING ARRAY IS NEEDED
100 	IF(non2p-1 > 0)  GOTO   200
    	np2hf = np2/2
  	j = 1
  	DO  i2 = 1,  np2,  np1
		IF(j-i2 >= 0) GOTO 130
     		i1max = i2 +  np1-1
    		DO  i1 = i2,  i1max
      			DO  i3 = i1,  ntot,  np2
        			j3 = j +  i3-i2
        			tempr = datar(i3) 
        			tempi = datai(i3) 
        			datar(i3)  = datar(j3) 
        			datai(i3)  = datai(j3) 
        			datar(j3)  = tempr
        			datai(j3)  = tempi
      			END DO
    		END DO
130   		m = np2hf
140   		IF(j-m <= 0)  GOTO 150
     		j = j-m
   		m = m/2
    		IF(m-np1 >= 0) GOTO 140
150     	j = j +  m
   	END DO
   
   	GOTO 300

!     SHUFFLE DATA BY DIGIT REVERSAL FOR GENERAL N
200  	DO  i1 = 1,  np1
    		DO  i3 = i1,  ntot,  np2
      			j = i3
      			DO  i = 1,  n
        			IF(icase-3 == 0) GOTO 220
    				workr(i)  = datar(j) 
        			worki(i)  = datai(j) 
        			GOTO 240
220   				workr(i)  = datar(j) 
        			worki(i)  = 0.
240   				ifp2 = np2
        			IFQ = ifmin
250   				ifp1 = ifp2/ifact(IFQ) 
        			j = j +  ifp1
        			IF(j-i3-ifp2 < 0) GOTO 260
    				j = j-ifp2
        			ifp2 = ifp1
        			IFQ = IFQ +  1
        			IF(ifp2-np1 > 0) GOTO 250
260   			END DO
      			i2max = i3 +  np2-np1
      			i = 1
      			DO  i2 = i3,  i2max,  np1
        			datar(i2)  = workr(i) 
        			datai(i2)  = worki(i) 
        			i = i +  1
      			END DO
       		END DO
      END DO

  
!     SPECIAL CASE--  W = 1
300 	i1rng = np1
	IF(icase == 2) i1rng = np0 * (1 +  nprev/2) 
    	IF(ntwo-np1 <= 0)  GOTO 600
    	DO  i1 = 1,  i1rng
    		imin = np1 +  i1
    		istep = 2 * np1
    		GOTO 330
310   		j = i1
    		DO  i = imin,  ntot,  istep
      			tempr = datar(i) 
      			tempi = datai(i) 
      			datar(i)  = datar(j) -tempr
      			datai(i)  = datai(j) -tempi
      			datar(j)  = datar(j)  +  tempr
      			datai(j)  = datai(j)  +  tempi
      			j = j +  istep
    		END DO
    		imin = imin +  imin-i1
    		istep = istep +  istep
330   		IF(istep-ntwo <= 0)  GOTO 310
    
!     SPECIAL CASE--  W = -SQRT(-1) 
    		imin = 3 * np1 +  i1
    		istep = 4 * np1
    		GOTO 420
400 		j = imin-istep/2
    		DO  i = imin,  ntot,  istep
    			tempr = datai(i) 
      			tempi = -datar(i) 
    			datar(i)  = datar(j) -tempr
      			datai(i)  = datai(j) -tempi
      			datar(j)  = datar(j)  +  tempr
      			datai(j)  = datai(j)  +  tempi
      			j = j +  istep
   		END DO
    		imin = imin +  imin-i1
    		istep = istep +  istep
420 		IF(istep-ntwo <= 0)  GOTO 400
430 	END DO
  
!     MAIN LOOP FOR FACTORS OF TWO.  W = EXP(-2 * PI * SQRT(-1)  * M/MMAX) 
  	theta = -twopi/8.D0
  	wstpr = 0.D0
  	wstpi = -1.D0
    	mmax = 8 * np1
  	GOTO 540
500 	wminr = DCOS(theta) 
  	wmini = DSIN(theta) 
  	wr = wminr
  	wi = wmini
  	mmin = mmax/2 +  np1
  	mstep = np1 +  np1
  	DO  m = mmin,  mmax,  mstep
    		DO  i1 = 1,  i1rng
      			istep = mmax
      			imin = m +  i1
510   			j = imin-istep/2
      			DO  i = imin,  ntot,  istep
        			tempr = datar(i)  * wr-datai(i)  * wi
        			tempi = datar(i)  * wi +  datai(i)  * wr
        			datar(i)  = datar(j) -tempr
        			datai(i)  = datai(j) -tempi
        			datar(j)  = datar(j)  +  tempr
        			datai(j)  = datai(j)  +  tempi
        			j = j +  istep
      			END DO
      			imin = imin +  imin-i1
      			istep = istep +  istep
      			IF(istep-ntwo > 0) THEN
	  	  		GOTO 525
	   		ELSE
	      			GOTO 510
	   		ENDIF
525    		END DO
       		wtemp = wr * wstpi
       		wr = wr * wstpr-wi * wstpi
       		wi = wi * wstpr +  wtemp
    	END DO
    	wstpr = wminr
    	wstpi = wmini
    	theta = theta/2.D0
    	mmax = mmax +  mmax
540 	IF(mmax-ntwo <= 0)  GOTO   500
  	
  
!     MAIN LOOP FOR FACTORS NOT EQUAL TO TWO.
!     W = EXP(-2 * PI * SQRT(-1)  * (J2-I3) /IFP2) 
600 	IF(non2p-1 <= 0)  GOTO   700
    	ifp1 = ntwo
  	IFQ = inon2
610 	ifp2 = ifact(IFQ)  * ifp1
  	theta = -twopi / ifact(IFQ) 
    	thetm = theta * np1 / ifp1 
  	wstpr = DCOS(theta) 
  	wstpi = DSIN(theta) 
  	wmstr = DCOS(thetm) 
  	wmsti = DSIN(thetm) 
  	wminr = 1.D0
  	wmini = 0.D0
  	DO  j1 = 1,  ifp1,  np1
    		i1max = j1 +  i1rng-1
    		DO  i1 = j1,  i1max
      			DO  i3 = i1,  ntot,  np2
        			i = 1
        			wr = wminr
        			wi = wmini
        			j2max = i3 +  ifp2-ifp1
        			DO  j2 = i3,  j2max,  ifp1
          				twowr = wr +  wr
          				jmin = i3
          				j3max = j2 +  np2-ifp2
          				DO  j3 = j2,  j3max,  ifp2
            					j = jmin +  ifp2-ifp1
            					sr = datar(j) 
            					si = datai(j) 
            					oldsr = 0.D0
            					oldsi = 0.D0
            					j = j-ifp1
 620   						stmpr = sr
            					stmpi = si
            					sr = twowr * sr-oldsr +  datar(j) 
            					si = twowr * si-oldsi +  datai(j) 
            					oldsr = stmpr
            					oldsi = stmpi
            					j = j-ifp1
            					IF(j-jmin > 0) GOTO 620
    						workr(i)  = wr * sr-wi * si-oldsr +  datar(j) 
            					worki(i)  = wi * sr +  wr * si-oldsi +  datai(j) 
            					jmin = jmin +  ifp2
            					i = i +  1
          				END DO
          				wtemp = wr * wstpi
          				wr = wr * wstpr-wi * wstpi
          				wi = wi * wstpr +  wtemp
       				END DO
        			i = 1
        			DO  j2 = i3,  j2max,  ifp1
          				j3max = j2 +  np2-ifp2
          				DO  j3 = j2,  j3max,  ifp2
            					datar(j3)  = workr(i) 
            					datai(j3)  = worki(i) 
            					i = i +  1
          				END DO
        			END DO
      	  		END DO
      		END DO
      		wtemp = wminr * wmsti
      		wminr = wminr * wmstr-wmini * wmsti
      		wmini = wmini * wmstr +  wtemp
  	END DO
   	IFQ = IFQ +  1
   	ifp1 = ifp2
   	IF(ifp1-np2 < 0) GOTO 610
  

!     COMPLETE A REAL TRANSFORM IN THE 1ST DIMENSION,   N EVEN,   BY CONJUGATE SYMMETRIES.
700 	SELECT CASE (icase ) 
    		CASE (1) 
      			GOTO 900
    		CASE (2) 
      			GOTO 800
    		CASE (3) 
      			GOTO 900
    		CASE (4) 
      			GOTO 701
  	END SELECT
701 	nhalf = n
  	n = n +  n
  	theta = -twopi/n
	wstpr = DCOS(theta) 
	wstpi = DSIN(theta) 
  	wr = wstpr
  	wi = wstpi
  	imin = 2
  	jmin = nhalf
  	GOTO 725		
710 	j = jmin		
  	DO  i = imin,  ntot,  np2
    		sumr = (datar(i)  +  datar(j) ) /2.	! these are mixed mode equations 
    		sumi = (datai(i)  +  datai(j) ) /2.     ! (datar is real, sumr is dble)
    		difr = (datar(i) -datar(j) ) /2.	! which is OK in F90/F95
    		difi = (datai(i) -datai(j) ) /2.
    		tempr = wr * sumi +  wi * difr
    		tempi = wi * sumi-wr * difr
    		datar(i)  = sumr +  tempr
    		datai(i)  = difi +  tempi
    		datar(j)  = sumr-tempr
    		datai(j)  = -difi +  tempi
    		j = j +  np2
  	END DO
  	imin = imin +  1
  	jmin = jmin-1
  	wtemp = wr * wstpi
  	wr = wr * wstpr-wi * wstpi
  	wi = wi * wstpr +  wtemp
725 	IF(imin-jmin < 0)  GOTO   710
  	IF(imin-jmin > 0)  GOTO 740

    	DO  i = imin,  ntot,  np2
    		datai(i)  = -datai(i) 
  	END DO
740	np2 = np2 +  np2
  	ntot = ntot +  ntot
  	j = ntot +  1
  	imax = ntot/2 +  1
745 	imin = imax-nhalf
  	i = imin
  	GOTO 755
750 	datar(j)  = datar(i) 
  	datai(j)  = -datai(i) 
755 	i = i +  1
  	j = j-1
  	IF(i-imax < 0) GOTO 750
    	datar(j)  = datar(imin) -datai(imin) 
  	datai(j)  = 0.
  	IF(i-j >= 0) THEN
		GOTO 780
	ELSE
		GOTO 770
	ENDIF
765 	datar(j)  = datar(i) 
  	datai(j)  = datai(i) 
770 	i = i-1
  	j = j-1
  	IF(i-imin > 0) GOTO 765
    	datar(j)  = datar(imin)  +  datai(imin) 
  	datai(j)  = 0.
  	imax = imin
  	GOTO 745
780 	datar(1)  = datar(1)  +  datai(1) 
  	datai(1)  = 0.
  	GOTO 900
  
!     COMPLETE A REAL TRANSFORM FOR THE 2ND,   3RD,   ETC. DIMENSION BY CONJUGATE SYMMETRIES.
800  IF(nprev-2 <= 0)  GOTO 900
     DO  i3 = 1,  ntot,  np2
    	i2max = i3 +  np2-np1
    	DO  i2 = i3,  i2max,  np1
      		imax = i2 +  np1-1
      		imin = i2 +  i1rng
      		jmax = i3 +  i3 +  np1-imin
      		IF(i2-i3 > 0) THEN
			jmax = jmax +  np2
		ENDIF
    		IF(idim-2 > 0)  THEN
        		j = jmax +  np0
      			DO  i = imin,  imax
        			datar(i)  = datar(j) 
        			datai(i)  = -datai(j) 
        			j = j-1
     			END DO
      		ENDIF
     		j = jmax
      		DO  i = imin, imax,  np0
        		datar(i)  = datar(j) 
        		datai(i)  = -datai(j) 
        		j = j-np0
      		END DO
    	END DO
   END DO
  
!    	 END OF LOOP ON EACH DIMENSION

900 	np0 = np1
    	np1 = np2
    	nprev = n
    	
 END DO		! end of: DO  idim = 1,  ndim

   RETURN
   END SUBROUTINE FFT


