SUBROUTINE WriteImage (peakcol, peakrow, peakwidth, peakavgGray)

!       Image Quality Measure (IQM) Program                                                                                       
!       Copyright, THE MITRE Corporation 1992 - 2007                                                                                
!       see Data Rights Notice in HELP SUBROUTINE                                                                             


! called right after "997  END DO"  in MainCaller
! Writes to file: the entire imaage in PGM format, with a black or white boundary line 
! around the  square subimage that corresponds to the peak IQ
! Expected to be already defined in global_data: totwidth, totheight, bpp
! Parms carried in: peakcol, peakrow, peakwidth (upper left corner col,row of subimage & subimage width), peakavgGray
! image(col,row) is entire image, 2-D, int*2 array
! It was determined via testing that that there should NOT be any accounting for polarity (order, minusplus, ipolarity) 
! when writing-out as pgm format (tested with tiff, pgm , raw input images)
 

 USE GLOBALDATA	 	
	    
 INTEGER :: peakcol, peakrow, peakwidth, peakcol11, peakrow11 
 INTEGER(2) :: dumy2
 INTEGER(4) :: dumy4
 
 CHARACTER (LEN=1) :: LF
 CHARACTER(LEN=14) :: numstring
 CHARACTER (LEN=100) :: outname


! LOGICAL :: file_existence
! CHARACTER (LEN=5) :: stringcol, stringrow


 LF = CHAR(10)		! MacOSX & Unix linebreak, which is used for output pgm file

 peakrow11 = peakrow + 1      ! peakrow,peakcol from Maincaller are referenced to 0,0 because they refer to a submimage
 peakcol11 = peakcol + 1      ! but in this subroutine need to convert to 1,1 origin for compatibility 
			      ! with range of image(col,row), which starts at 1,1, not 0,0
					   
 outname = TRIM(imgID)//'_IQ.pgm'	! cannot change/add to beginning of imgID because it may contain pathname
					! then a new false pathname would be created which doesnt exist

! if peakIQ_xxx file already exists (previously generated), change name of new file slightly so previous file stays as is
! INQUIRE(file=outname, exist=file_existence)
! IF(file_existence) THEN
!	i = LEN(TRIM(outname))
!	outname = 'peakIQ+'//outname(8:i)
! ENDIF



!      WRITE (stringcol,'(I5)') peakcol        this would be used in subimage filename, if Only the subimage is
!      WRITE (stringrow,'(I5)') peakrow        written to a file, instead of entire image (but doesnt quite work as is)
!      IF(peakcol < 10) stringcol = '0000'//stringcol
!      IF(peakcol < 100) stringcol = '000'//stringcol
!      IF(peakcol < 1000) stringcol = '00'//stringcol
!      IF(peakcol < 10000) stringcol = '0'//stringcol
!      IF(peakrow < 10) stringrow = '0000'//stringrow
!      IF(peakrow < 100) stringrow = '000'//stringrow
!      IF(peakrow < 1000) stringrow = '00'//stringrow
!      IF(peakrow < 10000) stringrow = '0'//stringrow
!      outname = trim(outname)//stringcol//stringrow//'.pgm'


 IF(bpp < 9) THEN
	OPEN(unit=UWRT, file=outname, form='unformatted', access='transparent', action='write', status='new',iostat=ierr)	
	
	IF(ierr /= 0) THEN
		CLOSE(UWRT)
 		PRINT*, 'Cannot create this image file (already exists):  ', outname
		RETURN
	ENDIF
		
	line = 0  	 ! if avg gray of subimage >= 50 then black line will be most visible, otherwise use white line
	IF(peakavgGray < 50.) line = 255

  ! add PGM header code:   
	WRITE(numstring, fmt='(I0,1x,I0)') totwidth, totheight	    ! width&height but subimage is always square
	WRITE(UWRT) 'P5'//LF
	WRITE(UWRT) ADJUSTL(numstring)//LF
	WRITE(UWRT) '255'//LF


!       create 2 pixel wide black or white boundary line around the square subimage:
	j = peakwidth - 1
	DO irow = peakrow, peakrow+j
		DO icol = peakcol11, peakcol11+j			
			IF(irow==peakrow11 .or. irow==peakrow11+1 .or. irow==peakrow11+j-1 .or. irow==peakrow11+j &
			  .or. icol==peakcol11 .or. icol==peakcol11+1 .or. icol==peakcol11+j-1 .or. icol==peakcol11+j) THEN
				image(icol,irow) = INT(line,kind(dumy2))
			ENDIF
		END DO
	END DO		


!     Write entire image to file, with boundary line around peak IQ subimage:
	DO irow = 1, totheight
		DO icol = 1, totwidth			
			intpix = INT(image(icol,irow),kind(dumy4))		
			IF(intpix > 255) intpix = 255
			IF(intpix < 0) intpix = 0
			WRITE(UWRT) CHAR(intpix)
		END DO
	END DO		

ELSE
	! write 9 to 16 bpp image to file
ENDIF


CLOSE(UWRT)

END SUBROUTINE WriteImage

