      SUBROUTINE bitfinder () 
                                                                                                                                  
!       Image Quality Measure (IQM) Program                                                                                       
!       Copyright, THE MITRE Corporation 1992 - 2007                                                                                
!       see Data Rights Notice in HELP SUBROUTINE                                                                             

!	THIS ROUTINE IS USED MAINLY TO DECIPHER ENDIANESS & POLARITY OF  16 BPP IMAGES;
!	POLARITY OF 8 BPP IMAGES CAN USUALLY BE MORE EASILY OBTAINED FROM OTHER IMAGE
!	VIEWER SW, e.g., ImageJ, Irfanview, GraphicConverter, etc.

!	Completion of this Bitfinder subroutine displays image format info
!	needed to properly read images into IQM; after gathering the needed info from
!	the Bitfinder output file (Bitfile) and using it to construct the aux data
!	input file for IQM, then IQM can be run.

!       This subroutine determines "Endianess" (Big or Little endian) of the image.
!	It is noted that, by most definitions endianess pertains to Byte order, not bit order,
!	and endianess would therefore not have meaning for a 1 Byte pixel.  Here, however, we 
! 	use the term as a convenience applied to denote the black/white polarity of 1 Byte pixels,
!	(which is needed to read-in 8bpp images with the correct polarity); the term is therefore
!	applied to 1 Byte pixels as well as 2 Byte pixels.

!	The output file, "BitFile",  displays the gray values for the first n pixels
!	in the image (top row, columns 0 to n-1), under both  endianess assumptions
!	(big &r little).  As of IQM v6.2, the "startbit" parameter is set to constant=0 for all images.
!	ALL images are treated as either 8bpp or 16bpp, so, for example, a 12bpp
!	image having a true gray range of 0 to 4095 would be treated as 0 to 65535 range with gaps.

!	The user needs to compare the output file to an independent 
! 	assessment of the true pixel values, in order to identify whether it is
!	Big or Little endian (order = B or L) and, for 16bpp, the polarity:
!	regular polarity is white=65535, black=0; if image is reverse polarity
!       then a negative sign is attached to B or L

!	In earlier versions (before IQM v6.2) the starting bit was also determined
!	where start bit = 0 for 8bpp image and start bit = 0,1,2,..,or 8 for 9bpp to 16bpp image
!	However this turned out to be an unneccessary complication; the start bit can be set to 
!	constant = 0 for any 1byte/pixel or 2byte/pixel image.
!	This startbit = 0 may result in e.g., a 12bpp image, which should have a true range 
!	of 0 to 4095, now having a range of 0 to 65535, with 15 graylevel gaps every 16th gray
!	level. but this 'bit stretch' has no effect on any of the computations in IQM.

!       Apple Macintosh convention is Big_Endian (order = 'B')                                                      
!	PC convention is Little_Endian (order = 'L')   

!	Any 2 bytes/pixel (16bpp) image is read as Int*2 by IBITS, which has range: -32768 to +32767 
!	so 16bpp image may have negative values (even if image was stored as unsigned)
!	so this case is converted back to all positives, in 0 to 65535 range, because this is range
!	used by IQM computations (IQM performs this neg to pos, as necessary).


      USE globaldata
                                                                                                                                  
      CHARACTER(1) :: ch, color 
      INTEGER ::  pixbit(2,0:99,0:99)
      INTEGER(2) :: pix2(2,0:99), bpp2, startbit2
      INTEGER(1) :: hdr
      LOGICAL 	fileexists                                                                                  
       
                                                                                                                                   
      PRINT * , ' ' 
      PRINT * , ' ' 
      PRINT * , ' ' 
      PRINT * , ' ' 
      PRINT * , 'DETERMINE BIT "ORDER" OF IMAGE FORMAT - NEEDED BY IQM TO READ IMAGES ' 
      PRINT * , '   for  8bpp image, this is B or L'
      PRINT * , '   for 16bpp image, this is -B, B, -L, or L' 
      PRINT * , '   B & L loosely refer to Big_endian and Little_endian, respectively'
      PRINT * , '   all grayscale images are assumed to be packed into 1 or 2 bytes per pixel, so:'
      PRINT * , '      image with 1 to 8bpp is called 8bpp'
      PRINT * , '      image with 9 to 16bpp is called 16bpp'
      PRINT * , '   Formats such as TIFF, RAW, PGM can be handled, but not BMP (because image is store upside down)'
      PRINT * , ' ' 
      PRINT * , 'PROCEDURE:' 
      PRINT * , '   1) Select one sample image from your set of images that have same format' 
      PRINT * , '      (e.g., all 8 bpp images from a Windows TIFF formatter).' 
      PRINT * , '   2) Identify & record (offline) true graylevels of first few pixels' 
      PRINT * , '      in top row: col#1, col#2,..., col#10' 
      PRINT * , '   3) Compare true graylevels to output of this routine (Bitfile)'
      PRINT * , '      to identify "order" parameter needed by IQM (-B, B, -L, or L)'
      PRINT * , ' '
      PRINT * , 'CAUTIONS:'
      PRINT * , '   -To verify correct read: check values of pixels displayed while IQM runs.' 
      PRINT *, '    -IQM cannot correctly read-in pixels that are stored as:' 
      PRINT *, '      complex, real(floats), signed integers, or in any compressed format,'
      PRINT * , '     But program may still run (output will be wrong!).' 
      PRINT * , '   -For image with >8bpp, Bitfile output reports graylevels in 0 to 65535 range.'                                                                                                                               
      PRINT * , '    If you record true graylevel of, e.g., 12bpp image in 0 to 4095 range, '
      PRINT * , '    you will need to multply true graylevel to match to Bitfile output range:'
      PRINT * , '    (truegraylevel + 1) * (65536 / 2**n)        where n=12 in this example.' 
      PRINT * , ' '
      PRINT * , ' '
10    PRINT * , ' '
     
      IF(cmdline == 'N') THEN
      		PRINT * , '   enter 3 parameters:' 
      		PRINT * , 'ImageFilename       headerBytes      bpp(8 or 16)' 
      		READ *, imgID, header, bpp 
      		PRINT*, ' '
      ENDIF
      
      IF(bpp < 8) THEN
      		bpp = 8
		PRINT*, 'Note - image is treated as 8 bpp '
      ELSEIF(bpp < 16 .and. bpp > 8) THEN
      		bpp = 16
      		PRINT*, 'Note - image is treated as 16 bpp '
      ELSEIF(bpp >16) THEN
      		STOP 'Invalid bpp input; must be 8 or 16'
      ENDIF
    
      lastpixel = 9	! imagetype is gray
      increment = 1        
       
   IF(cmdline == 'N') THEN
        PRINT*, 'Is this a color image?  y/n '
        READ *, color
        IF(color=='y' .or. color=='Y') THEN
      	  PRINT*, 'Are color layers (i)nterleaved or (s)equential? '
          PRINT*, 'Interleaved, e.g., TIFF "chunky" format, pixel sequence: R1,G1,B1,R2,G2,B2,...Rn,Gn,Bn '
          PRINT*, 'Sequential, e.g., TIFF "planar" format, pixel sequence:  R1,R2,...Rn,G1,G2,...Gn,B1,B2,...Bn'
      	  PRINT*, 'enter  i  or  s'
	  READ*, ch
	  IF(ch=='i' .or. ch=='I') THEN
		lastpixel = 3*lastpixel		! imagetype is RGB
		increment = 3
          ENDIF
       ENDIF
   ELSE
   	color = 'N'	! Dont process color image if Command Line version
   ENDIF
                                                                                                                                     
      
      INQUIRE (File='BITFILE', Exist = fileexists) 
      IF (fileexists) then 
            OPEN (unit=7, File='BITFILE', status='OLD', position='APPEND')    
      ELSE
      	    OPEN (unit=7, File='BITFILE', status='NEW')
      ENDIF    
      WRITE(7,'(///)')	
      WRITE (7,*) '  bpp=', bpp, '    headerBytes=', header, '     ', imgID     
      IF(color=='y' .or. color=='Y') WRITE(7,*) 'Reported Gray values are for First (usually Red) color layer'                                                                                                                           
      
       
        ns = 0	! this used to be variable startbit, is now constant startbit = 0
!       bpp2 = int(bpp,kind(startbit2)) 
  
  
IF(bpp == 8) THEN  	       
        WRITE(7,*)'               order:'
	WRITE(7,*) ' col        B        L '
 	 OPEN (unit=4,file=trim(imgID),action='READ',form='unformatted',&
	 	access='direct',recl=1, err=30)
	 DO ij = 1, header
	 	read(4,rec=ij) ch
	 END DO	   
	 DO icol = 0, lastpixel, increment       
	    	 read(4,rec=(header+icol+1)) ch
            	 pixbit(2, 0, icol) = ichar(ch) 		! order=L
		 pixbit(1, 0, icol) = 255 - pixbit(2, 0,icol)   ! order=B
	print*, icol, pixbit(2, 0, icol), pixbit(1, 0, icol)
	 END DO  
	 CLOSE (4) 
  
 ELSE   ! 16 bpp image:
        WRITE(7,*)'                          order:'
	WRITE(7,*) ' col       -B        B            -L         L'
       DO  k = 1, 2 		! read as Big-Endian then Little-Endian  
            IF (k == 1) then
	       OPEN(unit=4,file=trim(imgID),action='read',form='binary',convert='BIG_ENDIAN',err=30)
            ELSE
	       OPEN(unit=4,file=trim(imgID),action='read',form='binary',convert='LITTLE_ENDIAN',err=30)
	    ENDIF
            DO  ih = 1, header 
               READ (4) hdr
	    END DO
            DO icol = 0,lastpixel, increment 
               READ (4) pix2(k,icol) 
            END DO 
            CLOSE (4)    
	    
!	    startbit2 = INT(ns,kind(startbit2))    

            DO icol = 0,lastpixel, increment	! run thru 10 columns of pixels in top row
!		pixbit(k,ns,icol) = IBITS (pix2(k,icol), startbit2, bpp2) 
		pixbit(k,ns,icol) = INT(pix2(k,icol)) 	! since startbit2=0 and bpp2=16, no longer need for IBITS intrinsic
		IF(pixbit(k,ns,icol) < 0 ) THEN
			 pixbit(k,ns,icol) = 65536 + pixbit(k,ns,icol)
		ENDIF
	    END DO                                                                                 	    
       END DO  
ENDIF       
       
       
	ij = 0	   
	DO icol = 0, lastpixel, increment 
	    jbig = pixbit(1,ns,icol) 
	    jlittle = pixbit(2,ns,icol)
            IF(bpp == 8) THEN
		WRITE(7,'(I5,2I10)') ij, jbig, jlittle
	    ELSE
		WRITE (7,'(I5,2I10,4x,2I10)') ij, 65535-jbig, jbig, 65535-jlittle, jlittle
	    ENDIF
	    ij = ij + 1
	END DO               
   
	
      
      CLOSE (7) 
      PRINT*, ' '
      IF(cmdline == 'N') THEN
      		PRINT*, 'Another Image? y/n'
      		READ*, ch
      		IF(ch=='y'.or.ch=='Y') go to 10
      ENDIF
      PRINT*, '  inspect BITFILE output file to identify order: -B, B, -L, or L' 
      PRINT*, '  by comparing output to true graylevels'    
      STOP
30    STOP   'Fatal problem:   CANNOT OPEN IMAGE FILE' 
      
      END SUBROUTINE bitfinder  
      
