      SUBROUTINE HVS (avgslope, maxpowerwedge)                                                                                                            
                                                                                                                                  
!       Image Quality Measure (IQM) Program                                                                                       
!       Copyright, THE MITRE Corporation 1992 - 2007                                                                             
!       see Data Rights Notice in HELP SUBROUTINE                                                                            
                                                                                                                                  
!       This subroutine:                                                                                                          
! *** Computes human visual system factor, multiplies by 2-D normalized                                                           
! *** power spectrum (with other factors), and sums to COMPUTE IQ.                                                               
! ***                                                                                                                             
! ***  IQ = Summation normalized, factorized 2-D power(i,j), normally over freq range:                                                                 
! ***        0.01 to 0.707107 cycles per pixelwidth                                                
! ***        If ccd pixel banding exists then power near energy                                                    
! ***        spike is not use in IQ summation (this may be commented-out).                                                                   
! ***                                                                                                                             
! *** The Human Visual System model is the square of a rotationally symmetric
! *** modulation transfer function representing the HVS, i.e.,                                                                                       
! ***                                                                                                                             
! ***               H(r) = ((.2 + .45r)*exp(-.18r))                                                                               
! ***                                                                                                                             
! *** where r is the number of cycles/degree subtended by the eye:                                                                
! ***                                                                                                                             
! ***      r = .0174536(viewdist/spot)sqrt[(i/xlength)**2 + (j/ylength)**2]                                                       
! ***                                                                                                                             
! ***             viewdist = pseudo distance (mm): eye-to-display                                                                 
! ***             spot = pseudo crt pixel width (mm)                                                                              
!		Defining HVS peak location along power spectrum cycles/pixelwidth axis:
!			(292.774*spot) / viewdist = cy/pixewidth of HVS peak
!			spot=0.6, viewdist=351.3288   sets peak at 0.50 cy/pixwidth
!			spot=0.2  viewdist=585.548	  sets peak at 0.10 cy/pixwidth                                                                                                             
! ***  Parameter AAA in this subroutine = parameter T in MTR-10921 & in Opt.Eng.                                                  
! ***  April'92 paper; where AAA = T = (2*viewdist) Tan(0.5 degrees) / spot                                                       
! ***                                                                                                                             
! ***  The IQ is also normalized by the number of data points (rxlength*rylength).                                               
! ***  Summary of Additional Variables:                                                                                           
! ***     radialfreq   = sqrt[(i/rxlength)**2 + (j/rylength)**2]                                                                  
! ***     cyperdeg     = cycles per degree (=r)                                                                                   
! ***     hvsmtf       = H(r) above                                                                                               
! ***     FREQMINLOCAL = minimum radial frequency used in summation (usually = FREQMIN)
! ***     FREQMAX      = maximum radial frequency used in summation, must be <= 0.707107
                                                                                                                                  
      USE globaldata
	       
      DOUBLE PRECISION :: dbl_iq
      CHARACTER(LEN=3) :: oddevenwidth                                                                                                                                                                                                                                                                
                                                                                                                                  
      rxlength = REAL(xlength) 
      rylength = REAL(ylength) 
      dbl_iq = 0.0D0
      AAA = .0174536 * viewdist / spot 
      FREQMINLOCAL = FREQMIN 
      IF ((avgslope > slopecut) .and. (jsensor/= 4) .and. (jsensor /= 5) .and. (jsensor /= 6)) FREQMINLOCAL = ADJFREQ 
	     
!  For NIIRS sensors, if blur is detected, IQ summation range is changed from freqmin to freqminlocal, 
!  but not for sensors 4,5 or 6 which do not compute NIIRS.
!  IQ magnitude chgs drastically with summation range change, which is only compensated for
!  when computing NIIRS, by using different IQ-to-NIIRS constants; 
!  e.g. for sensor 5 (digital camera) the output quality parameter is "IQ" (no NIIRS is computed)
!  but if 2 subimages are processed, where 1 has avgslope > slopecut and the other does not,
!  then taking the avg IQ would make no sense.
	  
	                                                
!  The DO loop IQ sums are effectively over complete 4 quadrant Fourier transform by using power computed 
!  in 2 quadrants and bilateral symmetrism property of real image transform.
!  The different boundary conditions for even vs. odd number of pixels in image width is taken into account.
!  e.g, for 4x4 image, the following asterisked powervalues would be used in IQ sum: 	
!			  col= -2	    -1		0		1		2
! 		row= 2      	9	     5*		2*		5*		9*
! 		row=1	     	12	     8*		1*		6*		10*
! 		row=0	     	11	     7*		3		7*		11*
! 		row=-1     	10	     6*		4*		8*		12*
! 		row=-2     	9	     5		2		5		9

!  for 5x5 image col=-2 & row=-2 are also asterisked in above example

!  Note that bottom row at -ylength/2 is always included in below DO loop IQ sums, 
!  but power values in this bottom row were previously set = 0. (in compower) if ylength is even,
!  so no further compensation is needed here

! Before entry into this subroutine power spectrum has already been weighted by noise filter
! (if applied) and by scale, now weight power spectrum by HVS:

      DO  i = 0, xlength/2 
           rxl = (i / rxlength) **2 
           DO  j = -ylength/2, ylength/2 
	   	  ryl = (j / rylength)**2
	   	  radialfreq = sqrt (rxl + ryl) 
		  cyperdeg = AAA * radialfreq 
              	  hvsmtf2 = ((0.2 + 0.45 * cyperdeg) * exp ( - 0.18 * cyperdeg) ) **2 
		  power(i,j) = power(i,j) * hvsmtf2
	    END DO
      END DO



! *********************************************************************************
IF(highblur < -98.) THEN	! use fake negative value for highblur in prefsfile to turn-on this option

  ! USE THIS CODE TO DIVIDE POWER SPECTRUM BY A COMPONENT MTF BEFORE COMPUTING FINAL IQ VALUE
  ! e.g., "take-out" the image degradation caused by the imaging system that produced the digital image.
  ! MTF must be in equational form and is assumed to be rotationally symmetric.
  ! Assuming MTF equation has frequency parameter in cy/mm, with Nyquist frequency = Nyq, 
  ! then cy/mm value to plug into MTF equation, corresponding to given radialfreq, is:
  ! 				cy/mm = (Nyq) * (radialfreq) / (0.5 cy/pixelwidth)
  ! NOTE that although 0.5 cy/pixelwidth corresponds to Nyquist freq of MTF, the MTF needs to be 
  ! computed out to:  sqrt(2) * 0.5 = 1.414214 * 0.5 = 0.7071068 cy/pixelwidth,
  ! so equation for MTF must not fall apart between Nyqfreq & 1.4Nyqfreq (extrapolate curve smoothly if necessary)
	  
      cymmFactor = 20./.5		! for 1000ppi scanner MTF
      DO  i = 0, xlength/2 
           rxl = (i / rxlength) **2 
           DO  j = -ylength/2, ylength/2 
	   		ryl = (j / rylength)**2
	   		radialfreq = sqrt (rxl + ryl) 			  
			cymm = cymmFactor * radialfreq		
		      ! Microtek Artixscan-2500f scanner MTF @ 1000ppi, avg of VpeakMTF & HpeakMTF:
			systemMTF = 1.103487E-4*cymm**4 - 2.043974E-3*cymm**3 + 9.511593E-3*cymm**2 &
				   - 4.491906E-2*cymm + 1.059219
			
			systemMTFsquared = systemMTF * systemMTF
			IF(systemMTFsquared < TinySngl) systemMTFsquared = TinySngl
			power(i,j) = power(i,j) / systemMTFsquared
	   END DO
      END DO
ENDIF
! *********************************************************************************


! Now compute IQ value from weighted power spectrum:
      
      IF(mod(xlength,2) == 0) THEN		
      		oddevenwidth = 'evn'	! xlength is even so exclude leftside column at -xlength/2
      ELSE
      		oddevenwidth = 'odd'	! xlength is odd so include leftside column at -xlength/2
      ENDIF
      
      ihalfx = xlength/2  
      DO  i = 0, xlength/2 
           rxl = (i / rxlength) **2 
           DO  j = -ylength/2, ylength/2 
	   	   ryl = (j / rylength)**2
	   	   radialfreq = sqrt (rxl + ryl) 
		   IF (radialfreq >= FREQMINLOCAL .and. radialfreq <= FREQMAX) THEN
			 	IF((i==0) .or. (i==ihalfx .and. oddevenwidth=='evn')) THEN 
				   	dbl_iq = dbl_iq + dble(power(i,j))
		     		 ELSE
		     	  		dbl_iq = dbl_iq + 2.D0 * dble(power(i,j)) 
			 	ENDIF
		   ENDIF
	   END DO
      END DO
           
                                                                                                               
                                                                                          
        
! SKIP following code that removes power spike near 0.5 cy/pixelwidth, ostensibly due to pixel banding.
! i.e., pixel banding has been detected when CCDBAND detection threshold is exceeded, then mcode(8)='8'
! But have seen some cases where real image components can be 'detected' as CCD banding,
! e.g., highway lane lines.
! Following is old, early 1990s code, not best way to do it, and not in conformance
! with boundary conditions established starting in code version 6.0

! ---------------------------------------------------------------------------------------------
!  Compute IQ when sensor PIXEL BANDING exists, by avoiding power spike in vicinity of 0.5 cy/pixwidth.                                                                    
!  This follows original (longer compute) psdiqm code                                                                               
!  i.e., do j=ylength/2, -ylength/2, -1  
!  because angle in each of two quadrants is needed.                                                                                                       
!  'maxpowerwedge' is wedge number containing max power, found in RingWedgeQuirks
 
!  500 spikeangle = 90. - (6. * maxpowerwedge - 6.) 
!      spikeangmax = spikeangle+spikeangledelta 
!     spikeangmin = spikeangle-spikeangledelta 
! outer:   DO  i = 1, xlength / 2 
!              rxl = (i / rxlength) **2 
! inner:       DO  j = ylength / 2, - ylength / 2, - 1 
!                  angle = atan (REAL(j) / REAL (i) ) * 57.29578 
!                  radialfreq = sqrt (rxl + (j / rylength) **2) 
!                  IF ((radialfreq > spikefreq).and.(angle < spikeangmax).and.(angle > spikeangmin)) THEN
!				EXIT inner
!                  ELSEIF (radialfreq >= FREQMINLOCAL .and. radialfreq <= FREQMAX) THEN 
!                               dbl_iq = dbl_iq + 2.D0 * dble(power(i,j) 
!                  ENDIF 
!              END DO inner
!      END DO outer
	  
! ***   Add power along y-axis (which is 90 degree angle):                                                                        
!      DO j = ylength / 2, 1, - 1 
!           radialfreq = j/ rylength 
!           IF ((radialfreq > spikefreq).and.(90. < spikeangmax).and.(90. > spikeangmin)) THEN
!		 EXIT 
!           ELSEIF (radialfreq >= FREQMINLOCAL .and. radialfreq <= FREQMAX) THEN 
!                dbl_iq = dbl_iq + dble(power(0, j)  
!                dbl_iq = dbl_iq + dble(power(0,- j) 
!           ENDIF 
!      END DO 
! ---------------------------------------------------------------------------------------------


   IQ = sngl(dbl_iq) / (rxlength * rylength) 
!    print*, rxlength, rylength, acpower, dcpower, IQ       
           
           
           
                                                                                                                                  
      END SUBROUTINE HVS  
	  
