      SUBROUTINE MAINCALLER ()                       
                                                                                                                                  

!       Image Quality Measure (IQM) Program                                                                                       
!       Copyright, THE MITRE Corporation 1992 - 2006                                                                                
!       see Data Rights Notice in HELP SUBROUTINE                                                                             
                                                                                                                                  
!       MAINCALLER is called by IQM; MAINCALLER calls all other subroutines                                                             
                                                                                                                                                                                                                                          
! **********************************************************************************************
! Starting with IQM v6.0, April 2001, Brenner's FFT routine is used; this is supposed to work on 
! rectangle as well as ANY size square image, but in IQM this has been constrained to 
! ANY size Square image only, until the kinks are worked out for processing of rectangle image.
! Currently, CREATEAUX & READAUX & OUTPUT only use image or subimage width()
! In this MAINCALLER, the height() parameter has been added, as a 'placeholder', 
! temporarily setting:  height() = width()
! When change to rectangle FFT is made, then must add real height to CREATEAUX, READAUX, OUTPUT
! **********************************************************************************************


! *** Overview of Operations Performed                                                                                            
! ***                                                                                                                             
! ***   1. Compute 2-D Fast Fourier Transform                                                                                     
! ***   2. Compute Power Spectrum = (FFT)(FFT)* = |FFT||FFT|                                                                      
! ***   3. Normalize Power Spectrum by Image Size & DC, AC, or total Power                                                     
! ***   4. Average Equi-frequency Power Spectrum Values -- Result is 1-D                                                          
! ***          power spectrum with higher SNR.  (1-D power spectrum is used                                                       
! ***          for diagnostics, and degradation detection/measurement, not for IQ value).                                                           
! ***   5. Average power spectrum values in 4 degree wedges -- result                                                    
! ***          is used for diagnostics and degradation detection/measurement.                                                     
! ***   6. Compute Single-number IQ Based on 2-dimensional Power Spectrum, weighted by                                                       
! ***          Human Visual System MTF and, if applicable, Noise Filter and Directional Scale.                                              
                                                                                                                                  
! ***   IQ is in units of CYCLES PER GROUND FOOT; to change to cycles per ground meter                                                      
! ***   invoke statement at beginning of OUTPUT subroutine:  IQ = IQ/0.3048                                                        
! ***   AND change all IQ-to-niirs slope & intercept values (in prefs file)                                                      
                                                                                                                                  
! *** An AUXDATA file with information about each image to be processed is                                                        
! *** required. This file can be automatically                                                                                    
! *** constructed with the aid of user prompt parameter inputs in CREATEAUX.                                                       
! *** The AUXDATA file contains at least 2 lines of data for each image.                                                           
! *** Additional lines, consisting of (col,row,width) coordinate sets,exist                                                 
! *** if 1 or more subimages is to be processed for a given image (a subimage                                                     
! *** is a square image area buried within the total image area).                                                           
! *** Comment lines in the AUXDATA file are denoted with a # in column 1.                                                         
! *** See comments in SCALE subroutine for definition of most AUXDATA file parameters.                                                 
! *** ID is the image file name                                                                                                   
! *** The image coordinate system is such that COL=1, ROW=1 is the upper left                                                     
! *** corner of the total image (thus a 1024x1024 image runs from col=1 to 1024,                                                  
! *** row=1 to 1024).  The row axis (vertical direction on displayed image) is taken to be                                                    
! *** positive downward; the column axis (horizontal direction on displayed image)                                             
! *** is taken to be positve to the right.                                                                                                       
! *** But note that a subimage has relative reference to entire image, so ULcorner of subimage
! *** would be 0,0 if it coincides with ULcorner of entire image.
! *** Summary of some of the Variables Used:                                                                                      
! ***  (see SCALE subroutine & prefs file for additional parameters)                                                   
! ***     image      = character*2, two-dimensional array used to hold entire image                                               
! ***     datar,datai = Real & Imaginary I/O arrays for FFT                                             
! ***     power      = 2-D frequency domain real array                                                                            
! ***     ring       = 1-D frequency domain real array                                                                            
! ***     IQ        = computed image quality measure value                                                                       
! ***     ringwidth  = width of annular frequency bands; should normally be >=1                                                   
! ***     form      = 'SQR2' for square, power-of-2 image;                                                                       
! ***                  'SQRN' for square, but nonpower-of-2 image;                                                                
!                      'RECT' for rectangular image.                                                                        
! ***     xlength    = # of pixels in width of image or subimage whose FFT is computed                                            
! ***     ylength    = # of pixels in height of image or subimage whose FFT is computed                                                                                                         
! ***     psntype    = type of normalization used; either DC, AC, or total power                                                                
! ***     noiseflag  = specifies type of noise filter (if any) to be used                                                         
! ***     dcM, acM, xMval, or highdcM = M of the equation: MlogIQ + B = NIIRS                                                          
! ***     dcB, acB, xBval, or highdcB = B of the equation: MlogIQ + B = NIIRS                                                          
! ***     id         = image name                                                                                                 
! ***     xcoord     = array of subimage upper left x coordinates                                                                 
! ***     ycoord     = array of subimage upper left y coordinates                                                                 
! ***     bpp        = bits per pixel of image                                                                                   
! ***     totalIQ   = sum of IQs for all the subimages within an image                                                          
! ***     numsubsfinal    = # of useable subimages; used with totalIQ to compute average IQ                                                                                                                                   
! ***     avgslope   = avgslope of 1-D logPower vs. frequency curve from r=.05                                                    
! ***                  to r=.25 (see RingWedgeQuirks subroutine)                                                                     
! ***     slopecut   = if avgslope is > slopecut, IQ sum switches to r=adjfreq to r=.7                                           
! ***     freqmin    = minimum frequency for IQ summation when avgslope<=slopecut                                                
! ***     adjfreq    = minimum frequency for IQ summation when avgslope>slopecut                                                 
! ***     detectlevel = if (10th ac band/ 32nd ac band) is less than detectlevel                                                  
! ***                   noise is present & Wiener filter is applied                                                               
! ***     wedgeratio = ratio of max wedge  to min wedge, max selected from                                                    
! ***                  true max, or wedge to either side of true max                                                           
! ***     xlowfreqslope = average slope of 1-D log Power vs frequency curve over                                                  
! ***                     ac bands 1 to 3 (<0.05 cy/pixwidth)                                                                     
! ***     highblur    = if xlowfreqslope > highblur                                                                               
! ***     highhaze   = if sqrt(ac/dc) contrast < highhaze                                                                         
! ***     highnoise  = if noiseratio < highnoise                                                                                  
! ***     highsmear  = if wedgeratio > highsmear                                                                                  
! ***     mcode      = code number (0-8) denoting type of possible image degradation                                              
                                                                                                                                  
  

                                                                                                                                                                                                                                                        
      USE GLOBALDATA
	  
	  
      REAL :: 	noisevar, noiseratio, look
	  
      INTEGER :: xc, yc, values(8), peakcol, peakrow, peakwidth 
                                                                                                                                 
      CHARACTER(len=250) :: linedump 
      CHARACTER(len=132) :: line, verboseline 
      CHARACTER(len=100) :: badfilelist (100), idimgA, ABname  
      CHARACTER(len=7) ::	layer
      CHARACTER(len=6) ::	NoiseFilterSource
      CHARACTER(len=5) ::	stringavgiirs, stringpeakiirs
      CHARACTER(len=4) :: 	shaper
	  
      CHARACTER(10) :: 	time
      CHARACTER(8) ::	date
      CHARACTER(5) ::	zone
	                         
      CHARACTER(len=3) ::	imagepair					! 6 parms for image pair computations 
 	  
      REAL ::  IQA(5000), IQB(5000), IQratioAB(5000)		
	  
      INTEGER :: ULcol(5000), ULrow(5000) 				
	  
      LOGICAL fileexists 



! *********************************************************************************
! IMAGE PAIR PROCESSING (switch-on by setting highhaze = -99. in prefs file)
!	if imagepair='YES', then assumes every 2 sequential images in auxdatafile are an
!	image pair (same image with different processing or quality)
!	then program computes IQratio from IQsubimageA / IQsubimageB 
!	For this paired subimage case, user input options are:
!			paired subimage: IQratio that is furthest from 1.0 using same subimage location for A & B
!			unpaired subimage: maxIQA / maxIQB, regardless of subimage location in A or B
!			peakpaired subimage: maxIQA / (IQB at same location as A)
!	
!	In prefs file, minsize & maxsquare must have same value, which should be less than 1/2 of image width
!	to automatically segment input image into grid of IQ values computed from constant size subimages.
!	This code has only been verified for sensor #4, which does Not compute NIIRS.
!	However, NIIRS Difference between image A & image B can be computed from IQratio:
!	    delta NIIRS = M*Log(IQratio) , where M is slope between NIIRS & Log(IQ) relation defined in prefs file
!   TO TURN PAIR PROCESSING ON, SET IN PREFSFILE:  highhaze = -99.  (highhaze just used to carry on/off switch)
	
	IF(highhaze > -98.) THEN
		imagepair = 'NO '
	ELSE
		imagepair = 'YES'
		IF(minsize /= maxsquare) STOP   '  minsize & maxsquare must have same value in prefs file'
		PRINT*, ' '
		PRINT*, ' '
		PRINT*, '  Compute change in IQ (as ratio) between image pairs.'
		PRINT*, '    e.g., 4 input images: A1,B1,A2,B2, then A1 is compared to B1, A2 to B2, etc.'
		PRINT*, '  Additional output file:    IQratio_Output'
		PRINT*, '  THIS OPTION IS ONLY SETUP FOR SENSOR #4'
		PRINT*, ' '
		PRINT*, 'ENTER 1, 2, or 3:'
		PRINT*, '1    Paired Subimage:     IQratio furthest from 1.0, same A,B subimage location'
		PRINT*, '2    Unpaired Subimage:   maxIQA / maxIQB, regardless of subimage location in A or B'
		PRINT*, '3    Peakpaired Subimage: maxIQA / (IQB at same location as maxIQA in A)'
		READ*, ImagePairOption
		INQUIRE (File = 'IQratio_Output', Exist = fileexists) 
        IF (fileexists) then 
                IF (iqmnametype == 0) then 
                     OPEN (UPAR, File='IQratio_Output', status='REPLACE')                                                     
                ELSE 
                     OPEN (UPAR, File='IQratio_Output', status='OLD', position='APPEND') 
        	ENDIF
	ELSE 
		OPEN (UPAR, File='IQratio_Output', status='NEW')                                                                 
        ENDIF 
		
		WRITE(UPAR,*) CHAR(12)
		CALL DATE_AND_TIME(date,time,zone,values)	
		WRITE (UPAR,2)  values(2), values(3), values(1), values(5), values(6), values(7), TRIM(iqmversion)		   
2       FORMAT('mdy: ',I2,'/',I2,'/',I4,'     hms: ',I2,':',I2,':',I2,'      IQM ',A)
		WRITE(UPAR, *)  'normalization = ', psntype  
		IF(ImagePairOption == 2) THEN
			WRITE(UPAR,*) 'IQratio for Unpaired Subimages'
			WRITE(UPAR,*) 'd_NIIRS....IQratio.......IQa.........cola.....rowa.....IQb.......colb...rowb'
		ELSEIF(ImagePairOption == 1) THEN
			WRITE(UPAR,*) 'IQratio for Paired Subimages'
			WRITE(UPAR,*) 'd_NIIRS....IQratio.......IQa.............IQb..........cola...rowa..#imgs..width'
		ELSE
			WRITE(UPAR,*) 'IQratio for PeakPaired Subimages'
			WRITE(UPAR,*) 'd_NIIRS....IQratio.......IQa.............IQb..........cola...rowa..#imgs..width'
		ENDIF
		IF (psntype == 'DC') THEN
           		IQ_Nslope = dcM 			! IQ_Nslope is used later to convert IQ to delta Niirs
      	ELSEIF(psntype == 'AC') THEN 
           		IQ_Nslope = acM 
     	ELSEIF(psntype == 'TC') THEN 
           		IQ_Nslope = tcM 
  	  	ENDIF

	ENDIF
! *********************************************************************************


  
  PRINT*, ' '
  PRINT*, 'Image Read Verification:'
  PRINT*, '      gray levels of 4 pixels, as read by IQM, are displayed:'
  PRINT*, '      toprow col#1, col#2,    bottomrow col#1, col#2 of Entire image'
  PRINT*, '      if these values are not correct, image was not read-in correctly.'
  PRINT*, 'If multiple subimages: only first subimage info (IQ, NIIRS) is displayed.'
  PRINT*, ' '
 
  numbadfile = 0        ! counter for images or corresponding auxdata that could not be read                                                
  layercount = 1
  imgpairnum = 0	! image counter for paired image processing option

  DO 999 ifile = 1, 2000000 	! MAIN LOOP: go thru all images listed in auxdatafile, up to 2 million  	
	   
	   imagenumber = ifile
	   
		!  Auxdatafile read for image#1, image#1 processed, then next image's data read & processed,etc.
		!  "imgID" & "imagetype" are also read in ReadAux, but are global variables
		!  RGB colorimage: 
		!		3 independent passes (red,green,blue) thru complete "DO 999" loop, except
		!  		that auxdatafile only read once for that color image (alternative of reading auxdatafile
		!  		3 times, using "Backspace", is much more complex due to structure of ReadAux subroutine)
		!		however, caution, approach used assumes that parms read in ReadAux are NOT
		!		re-defined in other parts of program.
		
		IF(layercount == 1 ) THEN
		  	 CALL READAUX (fl, gsd, alt, pixel, az, look, fwdtilt, sidetilt, gamma, reltivepsilon,&
              			    numsubs, acpitch, acroll, acyaw, xmagmin, imagenumber, iendrun)    	

			! following is final test for correct read of auxdata (jsensor range is known so used here as test)
			! this logic not in readaux because initial errors jump to rereads 
			IF(jsensor < 1 .or. jsensor > 20) THEN
				PRINT*, ' '
				PRINT*, 'Cannot Read AuxDataFile'
				PRINT*, 'check: existence, data format, linebreak type, tabs(not allowed)'
				STOP
			ELSEIF(jsensor > 5 .AND. jsensor < 16) THEN
				PRINT*, '**** Skipping this image (invalid sensor number):     ',imgID
				CALL BadImageList(layercount, layer, badfilelist, numbadfile)
				GOTO 999
			ENDIF

			IF(xmagmin < 0.) THEN
            			IF(ifile == 1) THEN
					STOP ' Fatal problem with first image, cannot be skipped over.'	
				ELSE
					PRINT*, '**** Skipping this image (AuxDataFile may not be set up correctly):     ',imgID
					CALL BadImageList(layercount, layer, badfilelist, numbadfile)
					GOTO 999
				ENDIF
		ELSEIF(numsubs > maxsubs) THEN		
			PRINT * , '**** too many subimages (>',maxsubs,'), skipping:  ', imgID 
            		CALL BadImageList(layercount, layer,badfilelist,numbadfile)
			GOTO 999
		ENDIF 	 
			 
		
		
		
		IF(imagepair == 'YES') THEN
			 IF(jsensor /= 4) STOP  '  Images in Image pairs option must use sensor #4 '
				imgpairnum = imgpairnum +1				
			  	IF(mod(imgpairnum,2) /= 0) THEN			! imgpairnum is odd, i.e., first image of pair
					idimgA = imgID
					IF(ImagePairOption == 1) THEN		
						IQA = 1.0		
						IQB = 1.0 	
					ELSE
						IQA = 0.0		
						IQB = 0.0 
					ENDIF
			    		ULcol = 0
					ULrow = 0
			  	ENDIF
			  ENDIF	  
		
			  IF(iendrun == 1) EXIT		! exit DO 999 loop
			  CALL UPPERCASE(imagetype)
			  height = width		! for SQUARE FFT processing Only
		ELSEIF (layercount > 1 .and. imagetype(1:4) == 'GRAY') THEN
			  layercount = 1
			  GO TO 999
		ELSEIF (layercount > 3 .and. imagetype(1:3) == 'RGB') THEN
			  layercount = 1
			  GO TO 999		  
	    	ENDIF
		
  	    	IF(imagetype(1:3) == 'RGB') THEN
	  		IF(layercount == 1) layer = 'RED'
			IF(layercount == 2) layer = 'GREEN'
			IF(layercount == 3) layer = 'BLUE'
		ELSE
			layer = imagetype
	    	ENDIF   
		
		iflag = 0		! ValidSensors & allsensors defined in GlobalData Module	
		DO i=1,20
			IF (jsensor == ValidSensors(i) .and. jsensor /= 0 ) iflag = 1
			IF (jsensor > 15 .and. allsensors /= 'Y' ) iflag = 0
		END DO
        	IF(iflag == 0 .and. imagenumber > 1) THEN				        
			PRINT * , '**** Invalid sensor type, skipping:  ', imgID 
            		CALL BadImageList(layercount, layer,badfilelist,numbadfile)
			GOTO 999
		ELSEIF(iflag == 0 .and. imagenumber == 1) THEN
			STOP 'Fatal problem: first image has invalid sensor type'
		ENDIF

		numsubsfinal = 0
		IF(numsubs > 0 ) then	! store all subimage widths from auxdatafile, make sure none 
			jc = 0		! are > maxsquare;  numsubs=all subimgs, numsubsfinal=all useable subimgs
			DO jb = 1, numsubs
				IF(width(jb) <= maxsquare) then
					jc = jc+1
					tempwidth(jc) = width(jb)
				ELSEIF(imagepair == 'NO ') THEN
					PRINT*, 'Skipping this subimage, width > maxsquare (defined in Prefs file)'
				ELSE		! imagepair='YES', which is incompatible with defining subimage:
					PRINT*, 'Subimage was defined in auxfile but will be ignored,'
					PRINT*, 'because this is Paired Image processing run,'
					PRINT*, 'which operates on entire image.'
				END IF
			END DO
		   	numsubsfinal = jc				! numsubsfinal <= numsubs
			
			FORALL(jb=1:numsubsfinal)  width(jb) = tempwidth(jb)
			height = width		! for SQUARE FFT processing Only
		
		ENDIF

		   
		ALLOCATE (image(totwidth,totheight), stat=ierr)
		IF(ierr /= 0) THEN
			PRINT*, 'image memory allocation failed for:   ', imgID
			IF (imagenumber == 1) then 
            			STOP   'Fatal problem:   not enough memory available for first image'   
            		ELSE             
				CALL BadImageList(layercount, layer,badfilelist,numbadfile)
                		GOTO 999
			ENDIF 
		ENDIF
		
		iheader = header   
		itotwidth = totwidth
		itotheight = totheight
		IF(imagetype(1:3) == 'RGB') itotwidth = 3*totwidth
				
		CALL READIMAGE (itotwidth, itotheight, iheader, layer) 		! pixels stored in int*2 array: image(col,row)

		IF(totwidth == 0) then
			IF(imagenumber == 1) then
				PRINT*,   'Fatal problem with first image:  ',imgID
				STOP
			ELSE
				PRINT * , '**** Skipping this image, fatal problem    ', imgID 
                		CALL BadImageList(layercount, layer,badfilelist,numbadfile)
			    	GOTO 999
			ENDIF
		ENDIF  
                                                                                                                                  


! ***** DETERMINE IF ENTIRE IMAGE IS SQUARE-POWER-OF-2, General SQUARE, OR RECTANGLE                                                  
        IF (totwidth == totheight) then 
        	CALL POWER2 (totwidth, npower) 
		IF (npower > 0) then 
            		shaper = 'SQR2' 	! square power of 2 size				 
            	ELSE 
                	shaper = 'SQRN'		! square but not power of 2 size			 
            	ENDIF 
        ELSE 
            	shaper = 'RECT' 		! rectangle
        ENDIF 
                                                                                                                             
       
        totaliirs = 0.0 
        totalIQ = 0.0 
        totalcontrast = 0.0
	totalavgGray = 0.0
	denom = 0.0  
	avgiirs = 0.0
	peakiirs = 0.0
	peakIQ = -10.
	peakcol = 0
	peakrow = 0
	peakwidth = 0
		
  IF(jsensor /= 6) THEN			 
! ******  DETERMINE PROCESS FLOW:                                                                                                 
!        Process subimages that are defined in auxdatafile (numsubsfile > 0);                                                               
!        OR, if no subimages in auxdatafile: 
!        Process entire input image with single FFT (image must be square & width <= maxsquare)
!		 OR, if no subimages in auxdatafile and image is Rectangle or > maxsquare size:
!		 Call subimage to let program find all square subimages that fit in input image
!		 (initial numsubsfinal = 0)
        
        IF(numsubsfinal == 0) then
		IF (shaper == 'RECT') then 
            		CALL SUBIMAGE (numsubsfinal) 
		ELSEIF(totwidth > maxsquare) then
		   	CALL SUBIMAGE (numsubsfinal)
		ELSE	
		   	numsubsfinal = 1 
                	width (1) = totwidth 
                	height(1) = totheight
			xcoord (1) = 0 
                	ycoord (1) = 0  
		ENDIF
	ENDIF
	IF(numsubsfinal > maxsubs) THEN
		PRINT * , '**** too many subimages (>',maxsubs,'), skipping:  ', imgID 
            	CALL BadImageList(layercount, layer,badfilelist,numbadfile)
		GOTO 999
	ENDIF
  
  ELSE	
	IF(numsubsfinal == 0) THEN
		CALL BadImageList(numbadfile)
                GOTO 999 
        ENDIF
        
  ENDIF		




! ************************************************************************************************************
! ************************************************************************************************************
! ******  MAIN INNER LOOP, PROCESS EACH SQUARE SUBIMAGE OR WHOLE (SQUARE) IMAGE ******************************                                                                           
        DO 997 num = 1, numsubsfinal 	
            ylength = height (num) 
            xlength = width (num) 
            xnum = SQRT(REAL(xlength*ylength))
            denom = denom + xnum 
            xc = xcoord (num) 	
            yc = ycoord (num) 		! numpix,detnum only used for sensors 16,17,18 in SCALING
            numpix = detnum (num) 	! detnum is set=0 for other sensors in CREATEAUX & READAUX
            IF (shaper == 'RECT'.or.shaper == 'SQRN') numpix = detnum (1) 
            mcode = ' '			! mcode is char array, initialized to blank
            wedge = 0.		! for plot: unweighted wedge spectrum
            wedgeFilter = 0.	! for plot: wedge spectrum weighted by HVS filter,scaling,noise filter
            ring = 0.		! for plot: unweighted ring spectrum
            ringFilter = 0.	! for plot: ring spectrum weighted by HVS filter,scaling,noise filter
  
                                                                                                                                                                                                                                                                    
! ******* Convert 2-D int*2 subimage array to 1-D real array, DATAR(), for input to FFT                                                                      
!               'subimage' may equal entire image or may be true subimage
!               1-D real array is built-up row by row                                                  
!               DATAR() array runs 1.....n, origin ULcorner of subimage would be 0,0                                    	 

		ALLOCATE (datar(xlength*ylength), stat=ierr)
     		ALLOCATE (datai(xlength*ylength), stat=ierr)
     		kji = max(xlength, ylength)
     		ALLOCATE (workr(kji), stat=ierr)
     		ALLOCATE (worki(kji), stat=ierr)
		
		IF(ierr /= 0) then 
			PRINT*, '**** Memory allocation failed for FFT input array   ', imgID
                	CALL BadImageList(layercount, layer,badfilelist,numbadfile)
            		GOTO 999
		ENDIF 				 

				
! SETUP FOR INPUT TO FFT:  convert 2-D, int*2 subimage() array formed in ReadImage,  			
! to 1-D, real*4 datar() array.
! Range of int2 image() is -32768 to +32767, so 16bpp image may have neg values 
! but need all positive values for processing, so need to check for negs & shift to positive values
! Also, in some 16bpp cases the polarity may also be wrong i.e., white=0, black=65535, as signified
! by order = -B or order = -L; so reverse polarity in these cases.
		n=1
     		DO icol = (xc+1), (xc + xlength)	
     			DO irow = (yc+1), (yc + ylength)
				datar(n) = REAL(image(icol,irow))
				IF(datar(n) < -0.000001) THEN		
					datar(n) = 65536. + datar(n)
					IF(order == '-B' .or. order == '-L') datar(n) = 65535. - datar(n)
				ENDIF
				n=n+1
			END DO
     		END DO                                                                                                                   
            
			
			CALL FFT () 
 
			ALLOCATE (power(0:xlength/2, -ylength/2:ylength/2), stat=ierr) 
	
			IF(ierr /= 0) then
				PRINT*, '**** Memory allocation failed for power()        ', imgID
                		CALL BadImageList(layercount,layer,badfilelist,numbadfile)
            			GOTO 999
			ENDIF 
				
             CALL COMPOWER () 
            
	     DEALLOCATE (datar)
     	     DEALLOCATE (datai)
     	     DEALLOCATE (workr)
     	     DEALLOCATE (worki)
     	     
             CALL NORMALIZE (contrast, az, look, avgGray)                                                                                                                                
	
	     iFilter = 0
	     CALL RingWedgeQuirks (ringwidth, avgslope, wedgeratio, iFilter, &
			 	   xlowfreqslope, maxpowerwedge, smeardir, smearmag)       

             CALL WIENER (noiseratio, noisevar) 
              			                                                                                                                
             CALL SCALING (fl, gsd, alt, pixel, az, look, fwdtilt, sidetilt, gamma, reltivepsilon, &
                           true_range_gsd, acpitch, acroll, acyaw, xmagmin, numpix)                           
             
	     IF(xmagmin < 0.) THEN
            		PRINT*, '**** AuxdataFile not set up correctly for:    ',imgID
			CALL BadImageList(layercount, layer, badfilelist, numbadfile)
			GOTO 999
             ENDIF                                                                                                       
	     
	     CALL HVS (avgslope, maxpowerwedge)                                                                                                  

	     IF(IQ > peakIQ) THEN
			peakIQ = IQ
			peakcol = xc
			peakrow = yc
			peakcontrast = contrast
			peakavgGray = avgGray  
			peakwidth = xlength
	     ENDIF
 	     
 	     IF(imagepair == 'YES') THEN
 			IF(mod(imgpairnum,2) /= 0) THEN
				IQA(num) = IQ				! imgpairnum is odd, i.e.,first image of pair
			 	ULcol(num) = xc
				ULrow(num) = yc
			 ELSE
			 	IQB(num) = IQ
			 ENDIF
	      ENDIF
			
	      IF(lengthout == 'V') THEN
			iFilter = 1
			CALL RingWedgeQuirks (ringwidth, avgslope, wedgeratio, iFilter, &
			 			xlowfreqslope, maxpowerwedge, smeardir, smearmag) 
              ENDIF

	      DEALLOCATE (power) 
			      
	      
	   CALL OUTPUT (noiseratio, noisevar, contrast, avgslope, xlowfreqslope, wedgeratio,&   		                
             	          prediirs, xc, yc, xmagmin, ringwidth, imagenumber, avgGray,&
			  xMval, xBval, num, numsubsfinal, gsd, smeardir, smearmag, acpitch,layer)
   	               			
				                                                                                                                  
!       When subimages exist, final IQ is weighted sum of subimage IQs;                                                         
!       weighting is by subimage width (use of "denom" & "xnum")                                                            
!		i.e., finalIQ = SUM{(subimagewidth(i) x IQ(i)}  /  SUM{subimagewidth(i)}		(sum over all i)
!       There are at least 3 possible weightings that could be used when subimages exist:
!			simple average (no weighting), weighting by width, and weighting by area 
!		Experience/intuition suggests that weighting IQ & NIIRS quality by subimage width will more
!		closely follow how an observer would rate quality of the entire image, as a whole; as opposed
!		to the other two subimage quality weighting options.
!		However, the computed contrast is proportional to: standevGrayLevels / avgGrayLevel
!		
                totaliirs = totaliirs + xnum * prediirs 		! averages weighted by subimage widths
                totalIQ = totalIQ + xnum * IQ 				
		totalcontrast = totalcontrast + xnum * contrast		
		totalavgGray = totalavgGray + xnum * avgGray

997      END DO 		! end of processing for given subimage
      

	 minsize = tempminsize		! re-set to original values in case this is sensor #6 (which changes minsize,maxsquare)
 	 maxsquare = tempmaxsquare
	 
	 IF(minsize == maxsquare .AND. SubImageInfo(2:2) == 'W' .AND. numsubsfinal > 1) THEN
	 	CALL WriteImage (peakcol, peakrow, peakwidth, peakavgGray)
	 ENDIF

	 
	 DEALLOCATE (image)                                                                                                                       
             
        
  IF(numsubsfinal > 1 .OR. imagetype(1:3) == 'RGB' .OR. jsensor == 6) THEN   
!			OUTPUT AVG AND/OR PEAK IQ & IIRS, ALSO AVG OR PEAK CONTRAST FOR CASE OF MORE THAN 1 SUBIMAGE:
!			In IQM versions prior to v5.2, IIRS was deweighted if mcodes 2,3,4,or 5 were present,
!			so could not use avg_IQ to compute avg_iirs, so computed iirs for each subimage
!			and then computed avg_iirs from sample iirs values.
!			The following code still utilizes this separate computation of avg_IQ & avg_iirs
!			in case the iirs deweighting is reinstituted. 
!			(iirs deweighting is done in OUTPUT subroutine)
!			If mcode 1 is present, iq summation range (in HVS subroutine) is changed for
!			niirs sensors only (1,2,3) but not for non-niirs sensors (4,5); result is that
!			if 2 subimages, one with code=1, other with no code, then avg_iq for niirs sensor is
!			not too meangingul, but avg_iirs is fine; and avg_iq for non-niirs sensor is fine. 
      
!         	avgIQ = totalIQ / numsubsfinal			  ! simple arithmetic avg, all subimage sizes treated equally                                                                                         
!         	avgiirs = totaliirs / numsubsfinal 		  ! simple arithmetic avg                                                                                      
!		avgcontrast = totalcontrast / numsubsfinal 	  ! simple arithmetic avg    
            
			avgIQ = totalIQ / denom 						  ! avg weighted by subimage widths 
			avgcontrast = totalcontrast / denom				  ! avg weighted by subimage widths
			avgAvgGray = totalavgGray / denom
			IF(mcode(2)=='2'.or.mcode(3)=='3'.or.mcode(4)=='4'.or.mcode(5)=='5') THEN    
				avgiirs = totaliirs / denom		 
         		ELSE
				avgiirs = xMval * log10 (avgIQ) + xBval 
		 	ENDIF                                                                                                                                                                                                                                          
		 	IF(peakIQ /= 0.0) peakiirs = xMval * log10(peakIQ) + xBval
	 
	   		IF (imagetype(1:3) /= 'RGB' ) THEN		! multiple GRAY subimages
        	 		IF(jsensor==4 .or. jsensor==5 .or. jsensor==6) THEN
			 		IF(lengthout == 'D') THEN
			 			STRINGavgiirs = '-999.'
			 			STRINGpeakiirs = '-999.'
			 		ELSE
			 			STRINGavgiirs = '     '
						STRINGpeakiirs = '     '
					ENDIF
			 	ELSE
			 		WRITE (STRINGavgiirs,'(F5.2)') avgiirs
					WRITE (STRINGpeakiirs,'(F5.2)') peakiirs
			 	ENDIF
			 	IF(peakavg == 'PA') THEN
					WRITE (UOUT, 990) avgIQ, stringavgiirs, numsubsfinal,avgcontrast,avgAvgGray,jsensor,bpp,layer,pixelUL1,imgID 
990 			   		FORMAT (E13.6,2x,A5,'  avg:',I4,1x, F8.5,F8.1,20x,I2,2x,I2,1x,A7,I6,2x,A)     			   		
					WRITE (UOUT, 989) peakIQ, stringpeakiirs, numsubsfinal, peakcontrast,peakavgGray, peakwidth, peakcol,&
						                    peakrow, jsensor,bpp,layer,pixelUL1,imgID 
989			   		FORMAT (E13.6,2x,A5,' peak:',I4,1x, F8.5,F8.1, 3I7,2x,I2,2x,I2,1x,A7,I6,2x,A)						
			 		PRINT * , '        avg IQ =', avgIQ,'  ',trim(imgID)
			 		IF(lengthout == 'D') THEN
			 			WRITE (UDMP, 800) avgIQ, stringavgiirs, numsubsfinal,avgcontrast,avgAvgGray,jsensor,bpp,layer, pixelUL1,imgID
800 			   			FORMAT (E13.6,',',A5,',','avg of',I4,',', F8.5,F8.1,',-999,-999,-999,',I2,',',I2,',',A7,',' I6,',',A) 
						WRITE (UDMP, 801) peakIQ, stringpeakiirs, numsubsfinal, peakcontrast,peakavgGray, peakwidth, peakcol, peakrow,&
						                  jsensor,bpp,layer,pixelUL1,imgID 
801			   			FORMAT (E13.6,',',A5,',','peak from',I4,',',F8.5,F8.1,',',I7,',',I7,',',I7,',',I2,',',I2,',',A7,',',I6,',',A)						
			 		ENDIF
			 		
			 	ELSEIF(peakavg == 'P') THEN
		
					WRITE (UOUT, 989) peakIQ, stringpeakiirs, numsubsfinal, peakcontrast,peakavgGray, peakwidth, peakcol, peakrow,&
						                    jsensor,bpp,layer,pixelUL1,imgID 
					
					IF(lengthout == 'D') WRITE (UDMP, 801) peakIQ, stringpeakiirs, numsubsfinal,peakcontrast,peakavgGray,&
					 			      peakwidth,peakavgGray, peakcol, peakrow,jsensor,bpp,layer,pixelUL1,imgID 
				ELSE
					WRITE (UOUT, 990) avgIQ, stringavgiirs, numsubsfinal, avgcontrast, avgAvgGray, jsensor, bpp,layer, pixelUL1,imgID 
			 		PRINT * , '        avg IQ =', avgIQ,'  ',trim(imgID)
			 		IF(lengthout=='D') WRITE (UDMP,800) avgIQ, stringavgiirs, numsubsfinal, avgcontrast, &
			 						    avgAvgGray, jsensor, bpp,layer, pixelUL1, imgID
			 	ENDIF
				
				PRINT*, ' '
   			ELSE 	! color image must be in sequence: red, green, blue	
				IF(numsubsfinal == 1) THEN
					avgIQ = IQ
					avgcontrast = contrast
					avgAvgGray = avgGray
					avgiirs = prediirs
				ENDIF
				IF(layer == 'RED') THEN
					rgb_IQ = redW * avgIQ
					rgb_contrast = redW * avgcontrast
					rgb_avgGray = redW * avgAvgGray
					rgb_iirs = redW * avgiirs
				ELSEIF (layer == 'GREEN') THEN
					rgb_IQ = rgb_IQ + greenW * avgIQ
					rgb_contrast = rgb_contrast + greenW * avgcontrast
					rgb_avgGray = rgb_avgGray + greenW * avgAvgGray
					rgb_iirs = rgb_iirs + greenW * avgiirs
				ELSEIF(layer == 'BLUE') THEN
					rgb_IQ = rgb_IQ + blueW * avgIQ
					rgb_contrast = rgb_contrast + blueW * avgcontrast
					rgb_avgGray = rgb_avgGray + blueW * avgAvgGray
					rgb_iirs = rgb_iirs + blueW * avgiirs					
					IF(jsensor==4 .or. jsensor==5 .or. jsensor==6) THEN
			 			STRINGavgiirs = '     '
			 		ELSE
			 			WRITE (STRINGavgiirs,'(F5.2)') rgb_iirs
			 		ENDIF	
					WRITE (UOUT, 990) rgb_IQ,stringavgiirs,numsubsfinal,rgb_contrast,rgb_avgGray,jsensor,bpp,imagetype,pixelUL1,imgID 
        				PRINT * , '        avgRGB IQ =', rgb_IQ,'  ',trim(imgID) 	
					PRINT*, ' '
					IF(lengthout=='D') THEN
						IF(jsensor==4 .or. jsensor==5 .or. jsensor==6) STRINGavgiirs = '-999.'
						WRITE (UDMP,800) rgb_IQ,stringavgiirs,numsubsfinal,rgb_contrast,rgb_avgGray,jsensor,bpp,imagetype,pixelUL1,imgID
					ENDIF
				ELSE
					PRINT*, 'FATAL PROBLEM with color image, trilayer is not Red,Green,Blue      ',imgID
					STOP
				ENDIF	
    		ENDIF 
	ENDIF
		layercount = layercount + 1





! ************************************************************************************************		                		   
! ************************************************************************************************								   
! ************************************************************************************************
		IF(imagepair == 'YES' .AND. mod(imgpairnum,2) == 0) THEN
!				THIS ASSUMES EVERY TWO IMAGES IN INPUT SEQUENCE ARE A PAIR: IMAGE A & IMAGE B	
!				THIS COMPUTES RATIO BETWEEN IQ OF IMAGE A & IQ OF IMAGE B AND DELTANIIRS FROM IQ RATIO;
!				IS ONLY VERIFIED FOR USE WITH SENSOR #4, WHICH DOES NOT COMPUTE NIIRS
!				This imagepairs option is switched on by setting: highhaze=-99. in prefs file
!				also must have same value for minsize and maxsquare in prefs file
!				Originally coded for processing ---test images, ref: MITRE Tech Letter G034-L-311, 3/12/01, N.B.Nill
!				ALL additions to IQM code for this option are in this MainCaller subroutine.
! 				mod()=0 implies this is second image of pair (image B), so can now do pair calculations:
					peakIQA = 0.0
					peakIQB = 0.0
					ratiomin = 1.E30
					locationmin = 1
					locationmax = 1
					minflag = 0
					maxflag = 0
					numgoodimgs = 0
					ABname = ADJUSTL( TRIM(idimgA)//'   '//TRIM(imgID) )
					peakIQA = maxval(IQA)
					locationA = maxloc(IQA,1)
					peakIQB = maxval(IQB)
					locationB = maxloc(IQB,1)
					numgoodimgs = numsubsfinal
					IF(ImagePairOption == 1) THEN
							IQratioAB = IQA / IQB
							ratioHIGH = maxval(IQratioAB)
							ratioLOW = minval(IQratioAB)
							IF(ratioHIGH > (1./ratioLOW)) THEN
									FinalIQRatio = ratioHIGH
									locationA = maxloc(IQratioAB,1)
							ELSE
									FinalIQRatio = ratioLOW
									locationA = minloc(IQratioAB,1)
							ENDIF	
					ENDIF
						
					 
					 IF(ImagePairOption == 1) THEN
					 		IF(maxflag == 1 .and. minflag == 1) THEN
								IF(ratiomax > (1./ratiomin)) THEN
									FinalIQRatio = ratiomax
									locationA = locationmax
								ELSE
									FinalIQRatio = ratiomin
									locationA = locationmin
								ENDIF	
							ELSEIF(maxflag == 1 .and. minflag /= 1) THEN
								FinalIQRatio = ratiomax
								locationA = locationmax
							ELSEIF(minflag == 1 .and. maxflag /= 1) THEN
								FinalIQRatio = ratiomin
								locationA = locationmin
							ENDIF
					 		deltaNIIRS = IQ_Nslope * LOG10(FinalIQRatio)		
							WRITE(UPAR, 765) deltaNIIRS, FinalIQRatio, IQA(locationA), IQB(locationA), ULcol(locationA), &
							  	 ULrow(locationA), numgoodimgs, xlength, ABname 
					 
					 ELSEIF(ImagePairOption == 2) THEN
					 		FinalIQratio = peakIQA / peakIQB
					 		deltaNIIRS = IQ_Nslope * LOG10(FinalIQratio)		
							WRITE(UPAR, 764) deltaNIIRS, FinalIQratio, IQA(locationA), ULcol(locationA), ULrow(locationA), &
								 IQB(locationB), ULcol(locationB), ULrow(locationB), ABname
764					 		FORMAT(F7.2, F13.6, 2(E16.6, 2I7), 4x, A)
					 
					 ELSE
					 		FinalIQratio = peakIQA / IQB(locationA)
					 		deltaNIIRS = IQ_Nslope * LOG10(FinalIQratio)		
							WRITE(UPAR, 765) deltaNIIRS, FinalIQratio, IQA(locationA), IQB(locationA), ULcol(locationA), &
							      ULrow(locationA), numgoodimgs, xlength, ABname
765					 		FORMAT(F7.2, F13.6, 2E16.6, 4I7, 4x, A)	
					 ENDIF
					 							 	
		ENDIF
! ************************************************************************************************
! ************************************************************************************************
! ************************************************************************************************

999  END DO 		! end of processing for given image (with all its subimages)
                                                                                                                                  
       
	   IF(imagepair == 'YES') THEN
			PRINT*, '   2 OUTPUT FILES:    IQM_OUTPUT   &   IQratio_Output'                                                                  	
			WRITE(UPAR,*) ' '
			IF(imagepairoption == 1) THEN
				WRITE(UPAR, *) ' Paired Subimage:  IQratio furthest from 1.0, using same A,B subimage location'
			ELSEIF(imagepairoption == 2) THEN
				WRITE(UPAR, *) ' Unpaired Subimage:   maxIQA / maxIQB, regardless of subimage location in A or B'
			ELSE
				WRITE(UPAR, *) ' Peakpaired Subimage:   maxIQA / (IQB at same location as maxIQA)'
			ENDIF
			WRITE(UPAR, *) ' col  row  is upper left corner of subimage used in IQratio'
			WRITE(UPAR, *) ' IQratio > 1 implies image A is higher quality than image B'
			WRITE(UPAR, *) ' IQratio < 1 implies image B is higher quality than image A'
			WRITE(UPAR, *) ' slope for IQratio-to-deltaNIIRS is ', IQ_Nslope
			WRITE(UPAR, *) ' input auxdata file:  ', auxdataname
			WRITE(UPAR, *) ' input prefs file:    ', prefname
			WRITE(UPAR, fmt='(//)')
			CLOSE(UPAR)
	  ENDIF
		
		
! ***  Print degradation test codes and threshold values 
	  NoiseFilterSource = 'IQMstd'	! noiseflag=0
	  IF(noiseflag == 1) NoiseFilterSource = 'User'
      PRINT * , ' ' 
      WRITE (UOUT, 501) freqmin, freqmax 	  
501   FORMAT(/,'Problem Codes (thresholds set in Prefs file):',/, &
								  'blank   is normal image, freqmin =',F8.5,'    freqmax =',F8.5) 
      WRITE (UOUT, 503) adjfreq, slopecut 	  
503   FORMAT    ('  1   significant blur:   adjfreq =',F8.5,'   midfreqslope >',f10.4)                                                                                                                                                                  
      WRITE (UOUT, 505) highblur       
505   FORMAT    ('  2   severe blur:  lowfreqslope >',f10.3) 
      WRITE (UOUT, 507) highhaze    
507   FORMAT    ('  3   severe haze:    contrast <',f9.4) 
      WRITE (UOUT, 511) highsmear, wedgewidth      
511   FORMAT    ('  4   1-D smear:  wedgeratio >',f10.2, '  for wedge angle =',F8.3,' degrees') 
     
	  WRITE (UOUT, 513) NoiseFilterSource, highnoise       
513   FORMAT    ('  5   severe noise, ',A6,' Noise Filter Applied, noiseratio <',f9.3) 
      WRITE (UOUT, 515) NoiseFilterSource, detectlevel       
515   FORMAT    ('  6   significant noise, ',A6,' Noise Filter Applied, noiseratio <',f9.3)                                                                                                                                                                                                                                          
   
      WRITE(UOUT, 521) ccdband      
521   FORMAT    ('  8   sensor pixel banding: power(0.5cy/pix) >',f5.1,' * power(0.45cy/pix)')                                                                                                  
	  WRITE(UOUT,*) '  (problem codes not applied if image width < 33 pixels)'                                                                                                                              
	  WRITE(UOUT,*) ' '
	  
!  IF(smearexists == 'y') WRITE(UOUT,*) 'smear direction in degrees (+90 to -90 range); magnitude in pixelwidths' 

	  WRITE(UOUT,*) ' '                                                                                                                          
      	  WRITE(UOUT,*) 'Sensor:' 
	  WRITE(UOUT,*) ' 1     General Aerial/Space Digital Sensor'      
	  WRITE(UOUT,*) ' 2     Oblique Aerial/Space Digital Sensor'      
	  WRITE(UOUT,*) ' 3     Aerial/Space Film Camera'    
	  WRITE(UOUT,*) ' 4     General Sensor' 
	  WRITE(UOUT,*) ' 5     Ground-Based Digital Camera  (IQ >100 implies IQ dependent on # of sensor pixels)'                                                                                  
!	  WRITE(UOUT,*) ' 6     placeholder' 
	  
	  IF(allsensors == 'Y') then
	  	WRITE(UOUT,*) ' 16     T1'
	  	WRITE(UOUT,*) ' 17     T2'
	  	WRITE(UOUT,*) ' 18     T3'
	  ENDIF
	  WRITE(UOUT,* ) ' ' 
          WRITE(UOUT,* ) '"avg: 3"       IQ, NIIRS, contrast are averages from 3 subimages;'
	  WRITE(UOUT,* ) '               for subimage info, in prefsfile set:  SubImageInfo = Y'                                                                                                                    
          WRITE(UOUT,* ) ' ' 
	  IF(imagepair == 'NO ') THEN
	  		WRITE(UOUT,* ) '"peak: 4"      peak IQ & NIIRS from 4 subimages; contrast is for this peak subimage'
	  		WRITE(UOUT,* ) ' '
	  ENDIF
	  WRITE(UOUT,* ) 'Image polarity check:  "pixel1"  must equal true graylevel of UpperLeftCorner pixel'
	  WRITE(UOUT,* ) '                       of entire image; if RGB image, pixel1 value is for Blue layer'
	  WRITE (UOUT, * ) ' '
	  
	  IF(pgmImageFlag == 1) THEN
	  	WRITE(UOUT,*) ' '
	  	WRITE(UOUT,*) 'Caution: polarity of some .pgm images are defined as "B" in AuxDataFile;'
	  	WRITE(UOUT,*) '         verify this is correct polarity for read-in image (check "pixel1" value),' 
	  	WRITE(UOUT,*) '         if not correct, change "B" to "L" in AuxDataFile.'
	  	WRITE(UOUT,*) ' '
	  	PRINT*, ' '
	  	PRINT*,  'Caution: polarity of some .pgm images are defined as "B" in AuxDataFile;'
	  	PRINT*,  '         verify this is correct polarity for read-in image (check "pixel1" value),' 
	  	PRINT*,  '         if not correct, change "B" to "L" in AuxDataFile.'
	 	PRINT*, ' '
	  ENDIF
	 
	  WRITE (UOUT, * ) ' ' 
	  WRITE(UOUT,* ) 'NIIRS = -999   implies NIIRS not applicable'

!	  WRITE (UOUT, * ) 'IQ conversions:  IQac = IQdc / C**2        IQtc = IQdc / (1+C**2)'
!	  WRITE (UOUT, * ) '         where:  C = contrast,     IQtc = total power normalized IQ '
!	  WRITE (UOUT, * ) ' '
	  WRITE (UOUT, * ) ' '

     	 eyeratio = spot/viewdist
	 IF (freqmin<0.0099 .or. freqmin>0.0101 .or. freqmax<.707 .or. eyeratio<.16907E-02 .or. eyeratio>.17249E-02) THEN
	 		WRITE(UOUT,*) 'WARNING - Computed NIIRS is not correct, IF using DefaultPrefs M & B values, because'
	 		WRITE(UOUT,*) '          these values are only valid with DefaultPrefs: freqmin, freqmax, spot/viewdist;'
	 		WRITE(UOUT,*) '          but the value of at least one of these parms has been changed'	
	 ENDIF		
	  
	  
	  IF (numbadfile > 0) THEN
            WRITE(UOUT,*) '***********************************************************'
            WRITE(UOUT,*) 'Following labels were read as imagefilenames that could not'
            WRITE(UOUT,*) 'be processed, either because image file could not be found,'
            WRITE(UOUT,*) 'or auxdatafile not setup correctly, or other problem:'
    	    WRITE(UOUT, '(A100)' ) (badfilelist (iz), iz = 1, numbadfile)          
	  ENDIF 
          WRITE(UOUT,*) ' '                                                                                                                                                                                                                                                           
          
          
          IF (lengthout == 'V') THEN		! print verbose output
	  	   REWIND(UTMP)
		   DO i = 1, 1000000
		   	READ(UTMP,fmt='(a)',end = 526) verboseline
		   	WRITE(UOUT,fmt='(a)') verboseline
	  	  END DO		   
  		  CLOSE(UTMP)
  		  
526		WRITE(UOUT,528)
528   		FORMAT('plots of filtered spectra (after weighting by HVS & scale, and by noise filter,if applied')
	  		
		WRITE(UOUT,532) CHAR(12)
532   		FORMAT(A1,12x,'wedge angles:',/,6x,'max    >=   mid    >   min')
			
		DO i = 1, nint(180. / wedgewidth)
		  	ang  = 90. - (i-1) * wedgewidth
	 	  	angmax = ang + wedgewidth/2.
		  	angmin = ang - wedgewidth/2.
		  	WRITE(UOUT,fmt='(3(5x,F6.2))') angmax, ang, angmin
	  	END DO
		WRITE(UOUT, *) ' first wedge centered at +90 deg (+y axis in power spectrum domain)'
		WRITE(UOUT, *) ' wedge angles decrement clockwise'
        
		OPEN (UTMP, file = prefname, action = 'READ') 
           	WRITE (UOUT, 527) CHAR(12), prefname 
527 	   	FORMAT (//,A1,'  PREFERENCES DATA FILE:    ',A,/) 
           	DO i = 1, 500 
                	READ (UTMP, fmt='(A132)', end = 531) line 
         		WRITE (UOUT, fmt='(A132)') line 
 		END DO
531       	CLOSE (UTMP) 
        ENDIF 
	WRITE(UOUT,fmt='(////////)')                                                                                                                                                                                                                                                      
      


! *********** Write Prefsfile & Auxfile as comma-delimited data to DataDumpFile **********************************************    
      IF(lengthout == 'D') THEN
           	WRITE(UDMP,*) ' ' 
           	REWIND (USCH)
           	DO i = 1, 5000 			! Write AuxDataFile to comma-delimited file
         		READ (USCH, fmt='(A250)', end = 534) linedump
         		IF(i==2) THEN 
         			WRITE (UDMP, *) linedump,',',TRIM(auxdataname)
         		ELSE
         			WRITE (UDMP, *) linedump
         		ENDIF 
 		END DO 
534      	WRITE(UDMP,*) ' '		! Write PrefsFile to comma-delimited file
           	WRITE(UDMP,*) 'nrm,spot,view,minW,maxW,pkavg,subinfo,beginF,wedgeW,slpct,adjF,Fmin,&
                               &Fmax,hiblr,hismr,hihz,sighz,ccd,dcM,dcB,acM,acB,tcM,tcB,hidcM,hidcB,hiacM,hiacB,hitcM,hitcB,&
                               &red,grn,blu,Nflg,detlev,hiN,sigG2,rapw,sigS2,Dexp,K1,K2,Prefsfile,stdoutfile,stdfiletyp'
  		WRITE(UDMP,*) psntype,',',spot,',',viewdist,',',minsize,',',&
			      maxsquare,',',peakavg,',',subimageinfo,',',beginfreq,',',wedgewidth,',',slopecut,',',adjfreq,',',&
			      freqmin,',',freqmax,',',highblur,',',highsmear,',',highhaze,',',sighaze,',',ccdband,',',dcM,',',&
			      dcB,',',acM,',',acB,',',tcM,',',tcB,',',highdcM,',',highdcB,',',highacM,',',highacB,',',hightcM,&
			      ',',hightcB,',',redW,',',greenW,',',blueW,',',noiseflag,',',detectlevel,',',highnoise,',',sigmaG2,&
	             ',',rapw,',',sigmaS2,',',denomexp,',',kappa1,',',kappa2,',',TRIM(prefname),',',TRIM(iqmname),',',iqmnametype
      		WRITE (UDMP, fmt='(////)')
      		CLOSE (UDMP)
      		CLOSE (USCH)    		
      ENDIF
! *********** END of Prefs & Aux Data Write to DataDumpFile ************************************************************       
      
      
      
	  CLOSE(UTMP)
	  CLOSE (UOUT) 
      	  CLOSE (UAUX) 
      	
      	
      END SUBROUTINE MAINCALLER  
	  
