      SUBROUTINE normalize (contrast, az, look, avgGray) 
                                                                                                                                  
!       Image Quality Measure (IQM) Program                                                                                       
!       Copyright, THE MITRE Corporation 1992 - 2007                                                                                
!       see Data Rights Notice in HELP SUBROUTINE                                                                             
                                                                                                                                  
!       This subroutine:                                                                                                          
! *** Normalizes power by image size and DC power (DC normalizes out brightness),                                                 
! *** or normalizes power by image size and AC power (AC normalizes out haze, contrast).                                                    
! *** Also computes a measure of image contrast = Sqrt(AC/DC).                                                                                 
! *** If sensor=5 (digital camera) & user has selected the option, will also 
! *** normalize by CCD detector array size.

!  the definition of DC in this program is: DC = (avgGray)**2  x  (width x height)**2
!  in '92 Opt.Eng paper section 3.3, DC = (avgGray)**2
!  the difference is taken into account in the program computations


      USE globaldata
	       
      
      REAL :: look
	  
      dcpower = DBLE(power (0, 0)) 
      acpower = 0.D0	  ! acpower,dcpower declared double in Module
      contrast = 0.0
      rxry = REAL(xlength) * REAL(ylength)
	 
	  
                                                                                                                                  
! *** Compute AC Power from Total Power & DC Power:                                                                                                           	  
      totpower = SUM(DBLE(power(0, -ylength/2 : ylength/2)))	! along (y) vertical axis at x=0
      DO i = 1, xlength / 2 
           DO  j = - ylength / 2, ylength / 2   
                  totpower = totpower + 2.D0 * DBLE(power(i, j))   
	   END DO 
      END DO 
      acpower = totpower - dcpower
	
	
      avgGray = SNGL(DSQRT(dcpower))/rxry

		                                                                                                                          
! *** CONTRAST   (this value can be > 1.0):                                                                                                           
	  IF (dcpower < TinyDble) THEN
			contrast = 0.0
	  ELSE
	  		contrast = SNGL(DSQRT(acpower / dcpower))
	  ENDIF
	  
	  		   
		                                                                                                                          
! *** NORMALIZE POWER BY IMAGE SIZE                                                                                              
!	  Standard case is to normalize power by the size of the image, "rxry", whose FFT was 
!	  computed, i.e., normalize by:  #_of_width_pixels  x  #_of_height_pixels
!	  This makes it possible to compare quality levels from different size images.
!	  For this case:     power(i,j) = power(i,j) * rxry 
! 	  (Normalization by size multiplier here, whereas Opt.Eng.'92 paper shows it as divisor, but they are
! 	   equivalent - difference is taken into account via other normalizations in this program)

!	  However, for special case of digital camera evaluation (jsensor = 5)
!	  the number of pixels in the camera CCD array, "CCDsize", may be directly proportional 
!	  to the applicable definition of image quality, i.e., larger sensor array
!	  indicates higher image quality (all other factors being equal); in this case we
!     do Not want to normalize by array size. 
!	  To cover this case, user has option to NOT normalize by image size.
!	  if this option is invoked:     power(i,j) = power(i,j) * rxry * CCDsize	  
!	  Code is setup such that if subimages exist the normal case of normalizing
!	  power by the subimage size is first performed, as in standard case, 
!	  but then the subimage is un-normalized by the total image size = CCDsize.
! 	  The CCDwidthpixels, CCDheightpixels are stored in the auxdata inputfile 
!	  using the parameter names: az, look, respectively.
! 	  For jsensor=5, az=0., look=0. unless user option to un-normalize by
!	  CCDsize has been invoked.
	  	
	IF(jsensor == 5 .and. az > .9 .and. look > .99) THEN
		power = power * rxry * az * look	! az*look = CCDsize 
	ELSE
		power = power * rxry
	ENDIF
	
		
! Normalize Power by AC, DC, or TotalPower:			                                                                                                                                                                                                      
          
       
          IF(psntype == 'AC') THEN
	  		xnormalizer = SNGL(acpower)
	  ELSEIF(psntype == 'DC') THEN
	  	        xnormalizer = SNGL(dcpower)
	  ELSEIF(psntype == 'TC') THEN
	  		xnormalizer = SNGL(totpower)
	  ELSE
	  		STOP   'Quitting,  psntype in prefsfile must = AC, DC, or TC'
	  ENDIF   
    
     	  IF (xnormalizer < TinySngl) xnormalizer = TinySngl
     	  power = power / xnormalizer



! RESEARCH OPTION: do NOT normalize by image brightness, in this case run IQM with DC normalization
! and apply following line; takes out above dcpower normalization & renormalizes by image size
! (dcpower contains brightness normalization factor & image size normalization factor):
!	 power = (SNGL(dcpower)/(rxry*rxry)) * power     
! above is power = avgGray * avgGray * DCnormalizedPower
! Alternatively, if "no brightness normalization" is parameter in prefsfile, e.g., psntype=NB,
! then it is more direct to simply add NB branch to above IF clause. (and skip renormalization)
	
	  
       
! for TESTING only:	
! PRINT*, ' '
! PRINT*, 'normalized power spectrum, at end of normalize:'
!	DO irow = ylength/2, -ylength/2, -1
!		PRINT*, irow, (power(i,irow),i=0, xlength/2)
!	END DO
	   
	      
	                                                                                                                             
      
      END SUBROUTINE normalize   
	  
