SUBROUTINE PGM (kflag)
!       Image Quality Measure (IQM) Program                                                                                       
!     Copyright, THE MITRE Corporation 1992 - 2007                                                                               
!       see Data Rights Notice in HELP SUBROUTINE                                                                             
                                                                                                                                  
! This subroutine reads the image width & height in pixels, and header bytes,
! for a pgm-formatted image ("P5" format, 8bpp grayscale image)
! The image file can have MacOS9, MacOSX/Unix, or DOS/Windows linebreaks.
! Written by Norman Nill, Mitre, March 2003

! General P5 pgm format:
!		P5
!		# a comment
!		width  height
!		maxgray
!		pixel values...



  
    USE globaldata
    
    	CHARACTER(len=1) :: LF, CR
	CHARACTER(len=2) :: linend
	CHARACTER(len=4) :: tag
	CHARACTER(len=50) :: string
	CHARACTER(len=800) :: chunk		! assumes total header bytes < 800

	LOGICAL :: fileexists 
    
 	INTENT (OUT) :: kflag	!   0 = filename doesn't end in  .pgm   , so leave this subroutine
   				!   1 = filename ends in  .pgm  but file can't be read - Fatal Problem
   				!   2 = filename ends in  .pgm  and file is readable
 				! end-of-line:
 	LF = CHAR(10)		! Mac_OSX & Unix use LineFeed
	CR = CHAR(13)		! Mac_OS9 uses CarraigeReturn,     DOS/Windows uses CR followed by LF

 
 	j = LEN_TRIM(imgID)    
 	tag = imgID((j-3):j)	
 	CALL UpperCase(tag)
 	IF(tag /= '.PGM') THEN    
 		kflag = 0    
 		RETURN 		! subroutine is exited here if imagefilename doesn't end in  .pgm
 	ENDIF   

  	IF(tag == '.PGM' .and. imagetype(1:3) == 'RGB') GOTO 49
   	    
    INQUIRE (file = imgID, exist = fileexists)         
    IF (.NOT. fileexists) GOTO 55        
  
    OPEN (unit=UIMG, file=imgID, action='READ', form='unformatted', recl=800, access='direct', err=59)
	READ(unit=UIMG, rec=1, fmt='(A)', err=69) chunk

	IF(chunk(1:2) /= 'P5' .and. chunk(1:2) /= 'p5') GOTO 69

	IF(chunk(3:3) == LF) THEN
			linend = LF	
			ifirststart = 4
			itemp = 2
!			PRINT*, 'Mac OSX & Unix format'
	
	ELSEIF(chunk(3:3) == CR) THEN
			IF(chunk(4:4) == LF) THEN
					linend = CR//LF		
					ifirststart = 5
					itemp = 3
!					PRINT*, 'DOS/Windows format'
			ELSE
					linend = CR		
					ifirststart = 4
					itemp = 2
!					PRINT*, 'Mac OS9 format'
			ENDIF
	ELSE
			GOTO 89
	ENDIF
	

			iii=1
			istart = ifirststart
1			jjj = INDEX(chunk(istart:(istart+79)), TRIM(linend))  	! find next end-of-line after istart byte				
			iii=iii+1
			iend = (istart-1) + (jjj-1)
			IF(chunk(istart:istart) == '#') THEN	! line starting with # is just a comments line, skip it
				istart  = iend + itemp
				IF(iii > 40) THEN			!  if > 40 comment lines between P5 line & width,height line
					GOTO 93					!  then probably not true P5 pgm file
				ELSE 	
					GOTO 1
				ENDIF
			ELSE
				string = chunk(istart:iend)
				READ(string, *) totwidth, totheight
			ENDIF
			istart = iend + itemp		! this search assumes there are no more comment lines after width,height line
			jjj = INDEX(chunk(istart:(istart+79)), TRIM(linend)) 
			iend = (istart-1) + (jjj-1)
			header = iend + itemp - 1
!			PRINT*, 'header,width,height:  ', header, totwidth, totheight, imgID
			string = chunk(istart:iend)
			READ(string,*) maxgray
			IF(maxgray > 255) GOTO 95


   	
   	CLOSE(UIMG)
   	kflag = 2
   	bpp = 8   	

! 	correct order appears to be 'L' for most or all? pgm images, so if auxdatafile contains 'B' this flag will
!	notify user of possible polarity read-in problem (see MainCaller for notice)
	IF(order == 'B' .OR. order == 'b') pgmImageFlag = 1	
   	
   	RETURN		! return here when everything is OK

   	
   	
49  PRINT*, 'pgm file must be type: GRAY   ,but is type:  RGB '
	GOTO 99

55  PRINT*, 'cannot find the pgm image file:  '
    GOTO 99

59  PRINT*, 'pgm image file open failed'
	GOTO 99

69  PRINT*, 'pgm image file header cannot be read, e.g., does not begin with P5'
	GOTO 99

89  PRINT*, 'pgm image file linebreak type not recognized (is not Unix, Mac, or Windows type)'
	GOTO 99

93  PRINT*, 'more than 40 comment lines encountered, assumed not to be true pgm file'
	GOTO 99
	
95  PRINT*, 'maxgray in header > 255, is not true 8bpp grayscale pgm image'
	GOTO 99
	
99  CLOSE(UIMG)
 	kflag = 1
 	PRINT*, '**** Skipping this image (filename ends in .pgm but cannot read it): ',imgID
 	RETURN

   	END Subroutine PGM
  
  
  
