      SUBROUTINE readaux (fl, gsd, alt, pixel, az, look, fwdtilt, sidetilt, gamma, reltivepsilon, &
	  		  numsubs, acpitch, acroll, acyaw, xmagmin, imagenumber,iendrun) 
                                                                                                                                  
!       Image Quality Measure (IQM) Program                                                                                       
!       Copyright, THE MITRE Corporation 1992 - 2007                                                                                
!       see Data Rights Notice in HELP SUBROUTINE                                                                             
                                                                                                                                  
!       This subroutine:                                                                                                          
! *** Reads in the auxiliary data file, one image at a time;                                                                                          
! *** The AuxDatafile contains data about each image to be processed.                                                                     
! *** For each image there are at least 2 lines of data:
! *** first line contains image file name and "GRAY" or "RGB",
! *** second line contains image size info and sensor parameter values                                                  
! *** If subimages are to be processed for the given image, then there is                                                        
! *** one additional line for each subimage  (size & coordinates of subimage).                                               
! *** Comment lines begin with a '#' character IN FIRST COLUMN!, blank lines are OK.                                              

! *** This subroutine is complex because it overcomes some types of AuxDataFile corruptions,e.g.:
! ***       image file that can't be found
! ***       out-of-order data lines
! ***       wrong number or type of parameter values 
! *** If file corruption is detected for a given image, that image is skipped (not processed)
! **  then depending on the exact type of corruption, program will/will not continue with next image.
                                                                                                                               

                                                                                                                                  
      USE GLOBALDATA
	  
      REAL :: 	fl, gsd, alt, pixel, az, look, fwdtilt, sidetilt, gamma, reltivepsilon 
      REAL ::	acpitch, acroll, xmagmin 
	  
      INTEGER ::	auxflag 
      
      CHARACTER(LEN=200) :: lastline, curline
      
	 
      SAVE lastline, auxflag 
                                                                                                                                  
                                                                                                                                  
      numsubs = 0 
      IF (imagenumber == 1) then 			
           	auxflag = 0 
	   	iendrun = 0
	   	IF(lengthout=='D') WRITE(USCH,*) 'wdth,hght,hdr,bpp,odr,#px,fl,gsd,alt,pix,az,lok,fwd,sde,ga,&
	   			   &ep,pch,rol,yw,snsr,mod,mag,typ,imgID,auxfile'

      ELSEIF (auxflag == 1) then 
           	iendrun = 1		! iendrun=1 implies last image in file
		RETURN
      ELSE     	   
      	   IF(lastline(1:1) /= '#'.and. lastline(2:2) /= '#'.and. LEN(TRIM(ADJUSTL(lastline))) /= 0) THEN
5	      		CALL NameType(lastline)
	  		IF(imagetype(1:3) /= 'RGB' .and. imagetype(1:4) /= 'GRAY') THEN		! this works but doesnt flag
	  			READ (UAUX, fmt = '(a)', end = 260) curline			! the bad image, just skips it
	  			lastline = curline	
	  			GOTO 5
			ELSE
				GOTO 20
			ENDIF
	  ENDIF
      ENDIF 
              
                                                                                                                                
10    READ (UAUX, fmt = '(a)', end = 260) curline 
      IF (curline(1:1) =='#'.or.curline(2:2) =='#' .or. len(trim(adjustl(curline)))==0 ) goto 10 	
      CALL NameType(curline)
      IF(imagetype(1:3) /= 'RGB' .and. imagetype(1:4) /= 'GRAY') GOTO 10 
20    READ (UAUX, fmt = '(a)', end = 260) curline 
      READ (unit=curline, fmt=*, err=25) totwidth,totheight,header,bpp,order,detnum(1),fl,gsd,alt,pixel,&
           az,look, fwdtilt, sidetilt, gamma, reltivepsilon, acpitch, acroll, acyaw, jsensor, jsensormode, xmagmin                                                                                                                                                                                                                                
     
      GOTO 30
      
!     if above read(unit=curline...) failed, try the old auxdata format (pre v6.2), which included the startbit parameter:
25    READ (unit=curline, fmt=*, err=270) totwidth,totheight,header,bpp,startbit,order,detnum(1),fl,gsd,alt,pixel,&
           az,look, fwdtilt, sidetilt, gamma, reltivepsilon, acpitch, acroll, acyaw, jsensor, jsensormode, xmagmin                                                                                                                                                                                                                                                      


! F0 write chops-off all decimal digits to right of decimal sign, so it is N.G.
!      IF(lengthout=='D') WRITE(USCH, 66) totwidth,totheight,header,bpp,order,detnum(1),fl,gsd,alt,pixel,&
!           		az,look, fwdtilt, sidetilt, gamma, reltivepsilon, acpitch, acroll, acyaw, jsensor,&
!           		jsensormode, xmagmin, TRIM(imagetype), TRIM(imgID)    
! 66	FORMAT(4(I0,','),A3,',',I0,',',13(F0.0,','),2(I0,','),F0.0,',',A,',',A)      


30      IF(lengthout=='D') WRITE(USCH,*) totwidth,',',totheight,',',header,',',bpp,',',order,',',&
      			detnum(1),',',fl,',',gsd,',',alt,',',pixel,',',az,',',look,',', fwdtilt,',',&
      			sidetilt,',', gamma,',', reltivepsilon,',', acpitch,',', acroll,',', acyaw,',',&
      			jsensor,',', jsensormode,',', xmagmin,',',imagetype,',',TRIM(imgID)
	k = 1
	

50    READ (UAUX, fmt = '(a)', end = 290) lastline  
!            i = SCAN(ADJUSTL(lastline(1:80)), '/')	commented-out as of v7.1; the following fmt=* read cannot read an imgID     
!            IF(i /= 0) GOTO 280		that contains forward slash: /, but at this point in subroutine the
!						lines being read are subimage lines or blank lines, i.e., no "imgID" is present
      IF(jsensor==16 .or. jsensor==17 .or. jsensor==18) THEN
	      READ (unit = lastline, fmt = *, err = 280) xcoord(k), ycoord(k), width(k), detnum(k) 
	      IF(lengthout=='D') WRITE(USCH,*) xcoord(k),',',ycoord(k),',',width(k),',',detnum(k)                                                                                                      
      ELSE
	      READ (unit = lastline, fmt = *, err = 280) xcoord(k), ycoord(k), width(k)
	      IF(lengthout=='D') WRITE(USCH,*) xcoord(k),',',ycoord(k),',',width(k) 
	      detnum(k) = 0
      ENDIF 
      numsubs = k 			
      k = k + 1 
      GOTO 50 
                                                                                                                                  

280	  CALL PGM(kflag)
	  IF(kflag == 1) THEN
	  	xmagmin = -1
	  	RETURN
	  ENDIF
                                                                                                                                 
!     Check subimages for valid size:                                                                                           
      IF(numsubs > 0) THEN
	  DO  i = 1, numsubs 
             IF (((xcoord(i) + width(i)-1) > totwidth) .or. ((ycoord(i) + width(i)-1) > totheight)) THEN                                                                                    
                  PRINT * , '**** Skipping this image, incorrect subimage width or height.  ', imgID
		  xmagmin = -1.               
		  RETURN
             ENDIF 
      	     CALL primecheck(width(i), iprimeflag)
	  END DO 
      ELSE
	  CALL primecheck(totwidth, iprimeflag)
      ENDIF
      
      RETURN 	! Normal return when no problems encountered, for all but last image
  
                                                                                                                                    
260   auxflag = 1 	! normal return for last image
      iendrun = 1
      RETURN 
 
290   auxflag =1	! at first glance this path doesnt seem needed, i.e., statement 50  
      RETURN		! could say: end=260, then delete this 290 path; But testing shows that
                        ! for some types of corrupted AuxDataFiles this path IS Helpful                                                                                                          
                                                                                                                                  
270   xmagmin = -1.	! error condition
      RETURN 
                                                                                                                                                                                                                                                                                                                                                 
      END SUBROUTINE readaux 
      
      
      
      
  SUBROUTINE NameType(line)

! pull out imagefilename and imagetype  (both on same line), separation could be via spaces or tab

	USE GLOBALDATA
	
	CHARACTER(LEN=200) :: line, subline
	CHARACTER(LEN=1) :: tab=CHAR(9)
	
	line = ADJUSTL(line)	
      	i = SCAN(line, tab)
	k = LEN(line)
      	IF(i == 0) THEN		
      		j = SCAN(line,' ')		! find first space
       		subline = TRIM(ADJUSTL(line(j:k)))
        ELSE
      		j = i		! for tab separation
      		subline = TRIM(ADJUSTL(line((i+1):k))) 
      	ENDIF 
	imgID = line(1:(j-1))
	CALL UpperCase (imagetype)
	imagetype = subline(1:4)	
	
 END SUBROUTINE NameType  			  			

      
      
      
      
      
      

      
      

      
      

      
      
      
      
      
	  
