      SUBROUTINE readimage (itotwidth, itotheight, iheader, layer)  
                                                                                                                                  
!       Image Quality Measure (IQM) Program                                                                                       
!       Copyright, THE MITRE Corporation 1992 - 2007                                                                                
!       see Data Rights Notice in HELP SUBROUTINE                                                                             
                                                                                                                                  
!       This subroutine:                                                                                                          
! 	 	Reads in the Entire image, stored in int*2, 2-D array: Image(col,row) 
!		Origin of entire image is at upper left corner of entire image at 1,1     
!		A subimage, however, has a Relative reference to entire image,
!		so if UL corner of subimage coincides with UL corner of entire image
!		then the subimage location reference is 0,0		
                                                                                           
!       IF 1 byte/pixel, fast read as character string; 2 bytes/pixel read as int*2                                                                      
!		Back in MAINCALLER, the int*2, 2-D array whole image or subimage is 
!		re-stored as 1-D real array for input to FFT
                                                                     
!		Int*2 has range: -32768 to +32767 so 16bpp image as read-in by readimage
!		may have negative values, this case is converted back to all positives 
!		in MainCaller, i.e., 0 to 65535 gray level range 
                                                                                 
                                                         
                                                                                                                                  
      USE GLOBALDATA
	 
	  INTEGER 	:: polarity, plusminus
	  INTEGER(2) :: dumy, startbit2, bpp2    
	  INTEGER(2), ALLOCATABLE, DIMENSION(:) :: temp
	                                                                                            
	  CHARACTER(7) ::	layer
	  CHARACTER(13) :: 	endian
	  CHARACTER(kind=1,len=(itotwidth*itotheight+iheader)), ALLOCATABLE, DIMENSION(:) :: wholeimage 
    	  CHARACTER(kind=1,len=(iheader)), ALLOCATABLE, DIMENSION(:) :: wholeheader 
    
   	  LOGICAL :: fileexists 
    
      INQUIRE (file = imgID, exist = fileexists)         
      IF (.NOT. fileexists) THEN
      		totwidth = 0
      		PRINT*, '**** cannot find file: ', imgID
      		RETURN
      ENDIF        

	! Use of the global variables: totwidth,totheight,header, will not work in above CHAR...
	! and, for color chunky format image, need to use 3*totwidth
	! so substituted local variables: itotwidth=3*totwidth, itotheight=totheight, iheader=header
	
	startbit = 0	! needed to be a variable before IQM v6.2 (early Absoft compiler), but now can be set to constant=0  
	
	CALL UPPERCASE(layer)
	CALL UPPERCASE(imagetype)
	IF(imagetype(1:3) == 'RGB') THEN
			increment = 3	
			IF(layer == 'RED') ibegin = 1 
			IF(layer == 'GREEN') ibegin = 2 	
			IF(layer == 'BLUE') ibegin = 3 
	ELSEIF(imagetype(1:4) == 'GRAY') THEN
			increment = 1
			ibegin = 1
	ENDIF
	
	
	bpp2 = int(bpp,kind(dumy))
	startbit2 = int(startbit,kind(dumy)) 
	IF(order(1:1) == '-') THEN
		IF(order(2:2) == 'b') order = '-B'		! -B, -L are negative polarity 16bpp cases
		IF(order(2:2) == 'l') order = '-L'
	ELSE
		IF(trim(order) == 'b') order = 'B'
		IF(trim(order) == 'l') order = 'L'
	ENDIF
	IF(order/='-B' .and. order/='B' .and. order/='-L' .and. order/='L') THEN
		totwidth = 0
		print*, order,'**** Invalid Order, must be B or L (8bpp) or  -B,B,-L, or L (16bpp)    ', imgID
		print*, '           Run menu option 3 for explanation'
		RETURN           
	ENDIF

	 
	  
 IF(bpp <= 8) then	! 8bpp IMAGE READ, STORE AS int*2, 2-D ARRAY
   					! int*2 because combination of char()....wholeimage  & open() 
					! appears to use 2bytes/element instead of 1byte/element
		ALLOCATE (wholeimage(1),stat=ierr) 
		IF(ierr /= 0) then 
			print*, '**** Memory allocation failed for wholeimage() array for: ', imgID
			PAUSE
			STOP
        	ENDIF 			

		OPEN (unit=UIMG,file=imgID,action='READ',form='unformatted',access='direct',&
		  	recl=(itotwidth * itotheight + iheader), iostat=ierr)	  
	  	
		IF(ierr /= 0) then
			CLOSE(UIMG)
			PRINT*, '**** Image file open error for:     ',imgID
			totwidth = 0
			DEALLOCATE (wholeimage)
			RETURN
		ENDIF

		READ(unit=UIMG,rec=1,iostat=ierr) wholeimage(1)
		IF(ierr /= 0) then
			totwidth = 0
			PRINT*, '**** Image read error for:    ',imgID
			CLOSE(UIMG)
			DEALLOCATE (wholeimage)
			RETURN
		ENDIF
	  	
		IF (order == 'B') then                                                                                              
			 polarity = 255
			 plusminus = -1
	  	ELSEIF(order == 'L') then
	  	         polarity = 0
		         plusminus = 1
	  	ENDIF   
		
	  	DO irow = 1, itotheight
	  	     nnn = 0
		     DO icol = ibegin, itotwidth, increment
			     j= iheader + (irow-1) * itotwidth + icol
			     ijk = polarity + plusminus * ICHAR(wholeimage(1)(j:j))
			     nnn = nnn+1
			     image(nnn,irow) = INT(ijk, kind(dumy))
		     END DO
	  	END DO
		DEALLOCATE (wholeimage)  
		   
		   
		   
 ELSEIF(bpp > 8.and.bpp <= 16 ) THEN	! 9-16bpp IMAGE READ, STORE AS int*2,2-D ARRAY 	 
	
		IF(order=='B' .or. order=='-B') then
			endian = 'BIG_ENDIAN'
		ELSEIF(order == 'L' .or. order=='-L') THEN
			endian = 'LITTLE_ENDIAN'
		ENDIF
		
		OPEN(unit=UIMG,file=imgID,action='read',form='binary',convert=trim(endian),iostat=ierr)

		
		IF(ierr /= 0) THEN
			CLOSE(UIMG)
			PRINT*, '**** Image file open error for:    ', imgID			
			totwidth = 0
			RETURN
		ENDIF
		
		ALLOCATE (wholeheader(1),stat=ierr) 
		IF(ierr /= 0) then 
			print*, '**** Memory allocation failed for wholeheader() array for: ', imgID
			PAUSE
			STOP
        	ENDIF 			
	
		READ(unit=UIMG,rec=1,iostat=ierr) wholeheader(1)		
		IF(ierr /= 0) then
			totwidth = 0
			PRINT*, '**** Image header read error for:    ',imgID
			CLOSE(UIMG)
			DEALLOCATE (wholeheader)
			RETURN
		ENDIF
		DEALLOCATE (wholeheader)	  			
				 	
		ALLOCATE (temp(itotwidth), stat=ierr)
		IF(ierr /= 0) THEN 
			PRINT*, '**** Image memory allocation failed for:    ', imgID
			STOP
        	ENDIF 		
	
	
		DO nr = 1, itotheight
						
	   		READ(unit=UIMG,iostat=ierr) (temp(j), j=1,itotwidth)	

			IF(ierr /= 0) THEN
				CLOSE(UIMG)
				DEALLOCATE (temp)
				PRINT*, '**** Image read error for:     ',imgID
				totwidth = 0
				RETURN
			ENDIF			
			
			nnn = 0
			DO ic = ibegin,itotwidth, increment
				nnn = nnn+1
					! no longer need IBITS since startbit2 always=0 and bpp2 is constant 16 (v7.3 change)
					! image(nnn,nr) = IBITS(temp(ic), startbit2, bpp2) 
				image(nnn,nr) = temp(ic)
			END DO   
        	END DO    	
       	
  		DEALLOCATE (temp)
 ELSE
   		print*, '****  bpp= ',bpp, ' ,which is out of range'
		totwidth = 0
		RETURN
 ENDIF
    

	
	! range of int2 image() is -32768 to +32767, so 16bpp image may have 
	! negative values in int2 image() format, but will be using all positive values
	! when image() is converted to Real datar() in MainCaller, 
	! so adjust printout values correspondingly:
	pixelUL1 = int(image(1,1))				! top row, col 1, col 2
	pixelUL2 = int(image(2,1))
	pixelLL1 = int(image(1,totheight)) 		! bottom row, col 1, col 2
	pixelLL2 = int(image(2,totheight))
	IF(pixelUL1 < 0) pixelUL1 = 65536 + pixelUL1 
	IF(pixelUL2 < 0) pixelUL2 = 65536 + pixelUL2 
	IF(pixelLL1 < 0) pixelLL1 = 65536 + pixelLL1                                                                                                                      
    	IF(pixelLL2 < 0) pixelLL2 = 65536 + pixelLL2 
	IF(order == '-B' .or. order == '-L') THEN
		pixelUL1 = 65535 - pixelUL1		! these are 16bpp negative polarity cases
		pixelUL2 = 65535 - pixelUL2
		pixelLL1 = 65535 - pixelLL1
		pixelLL2 = 65535 - pixelLL2
	ENDIF
		

! Check that image file size is consistent with header, width, height values (requires FLEN use):
!	 IF(bpp == 8 ) THEN
!	 	IF(imagetype(1:4) == 'GRAY') THEN
!	 		nbytes = header + totwidth * totheight 	
!	 	ELSE
!	 		nbytes = header + 3 * totwidth  * totheight		! rgb color image,
!	 	ENDIF
!	 ELSE
!	 	nbytes = header +  2 * totwidth * totheight		! gray image, 2bytes per pixel
!	 ENDIF
	  
!	 INQUIRE(unit=UIMG, FLEN = ifilesize)	! FLEN (filelength) is Absoft F95 extension
!	 IF(ifilesize > nbytes .and. imagetype(1:4) == 'GRAY') THEN	!  too many possibilites to include rgb
!	 	PRINT*, ' '
!	 	PRINT*, '**** Caution: ImageFile has',ifilesize-nbytes, ' more bytes than sum: (Headerbytes + Pixelbytes)     ', imgID
!	 	PRINT*, ' '      
!	 ENDIF

	
	CLOSE(UIMG)

END SUBROUTINE readimage  















					
			                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   
		
		
