      SUBROUTINE SUBIMAGE (numsubsfinal) 
                                                                                                                                  
!       Image Quality Measure (IQM) Program                                                                                       
!       Copyright, THE MITRE Corporation 1992 - 2007                                                                                
!       see Data Rights Notice in HELP SUBROUTINE                                                                             
                                                                                                                                  
!     This subroutine is used when the ENTIRE IMAGE is not square,
!	  or is square but is too large to process all at once.                                                                                                          

!     Locates most of the square, nonoverlapping subimages within                                                      
!     the area of the ENTIRE IMAGE, which may be square or rectangular. 
!	  Note that there can be a remainder area in lower right corner of
!	  ENTIRE image in which subimages are not located; this portion of image
!	  is not measured; end ofthis routine computes the relative area of
!	  this non-measured area.

!     The upper left column and row of each square subimage is found, together                                                    
!     with its width; the upper left column, row of the first subimage is always                                                  
!     at (0,0) not (1,1) and is always the largest square subimage possible.                                                             
!     minimum subimage size, "minsize" is defined in the Prefs file.                                                                      
!     maxsubs = maximum number of subimages that will be found (defined in Module in IQM subroutine)                                                        
!     totwidth = number of columns in ENTIRE IMAGE                                                                          
!     totheight = number rows in ENTIRE IMAGE                                                                               
!     xcoord(i) = upper left column of ith square subimage found                                                                         
!     ycoord(i) = upper left row of ith square subimage found                                                                            
!     width(i) = width of ith square subimage (in pixels)                                                                      
!     numsubsfinal = total number of square subimages found                                                                
!	  maximage = width of largest square image that fits in ENTIRE IMAGE
!				 except that maximage cannot be > maxsquare (defined in Prefs file)
                                                                                                                                  
                                                                                                                                  
      USE globaldata
	  
                                                                                                                     
	  xcoord = 0 		
          ycoord = 0 
          width = 0 
	  height = 0
          numsubsfinal = 0
 	 
!     Find maximage = width of max size square subimage that fits in input image:  	  
	  maximage = min(totwidth, totheight) 
      IF (maximage < minsize) then 
           PRINT * , 'Input image is smaller than', minsize, ' x', minsize, ' pixels.' 
           PRINT * , 'Input larger image or decrease "minsize" in Prefs file.'  
           STOP
	  ELSEIF(maximage > maxsquare) then
	  	   maximage = maxsquare		! maxsquare is max size image for FFT, defined in Prefs file
      ENDIF 	 
	 
                                                                                                                                
!     Locate all maximage x maximage subimages that fit in image:                                                  
     iWnum = int(totwidth/maximage)
	 iHnum = int(totheight/maximage)
	 numsubsfinal = iWnum * iHnum
	 DO j = 1, iHnum
	 	DO i = 1, iWnum
			kk = (j-1) * iWnum + i
            xcoord(kk) = (i-1) * maximage
			ycoord(kk) = (j-1) * maximage
			width(kk) = maximage
		END DO
	  END DO
           
                                                                                                                      
!     Locate all subimages along bottom remainder rows
!	  setting subimage width = number of remainder rows                                                                                                                        
      istartrow = ycoord (kk) + width (kk) 
 	  isubwidth = totheight - istartrow		
      IF (isubwidth >= minsize) THEN
      		n = totwidth/isubwidth
			DO j = 1, n 
                  numsubsfinal = numsubsfinal + 1
				  width (numsubsfinal) = isubwidth 
                  ycoord (numsubsfinal) = istartrow 
   		          xcoord (numsubsfinal) = (j - 1) * isubwidth 	  
			END DO
      ENDIF 


!     Locate all subimages along rightside remainder columns
!	  setting subimage width = number of remainder columns  
!	  (but do not overlap any existing 'bottom rows remainder' subimages)
	  istartcol = xcoord (kk) + width (kk) 
 	  isubwidth = totwidth - istartcol		
      IF (isubwidth >= minsize) THEN
      		n = totheight/isubwidth
		DO j = 1, n 
                  	IF((j*isubwidth) > istartrow) EXIT	! exit DO j=1,n loop
			 numsubsfinal = numsubsfinal + 1
			width (numsubsfinal) = isubwidth 
                  	ycoord (numsubsfinal) = (j - 1) * isubwidth
   		         xcoord (numsubsfinal) = istartcol	  
		END DO
      ENDIF 

	
	MeasuredArea = 100 * SNGL( SUM(dble(width)*dble(width)) / (dble(totwidth) * dble(totheight)) )
	PRINT*, ' '
	PRINT*, MeasuredArea,'% of total area measured for   ', imgID
	
	height = width 		! set all subimageHeights equal to corresponding subimageWidths
	
    
	DO i = 1, numsubsfinal
		CALL primecheck(width(i), iprimeflag)
	END DO
    
	
	END SUBROUTINE SUBIMAGE  
	


